@@ -18,8 +18,13 @@ namespace js {
 // active Debuggers.
 
 class AbstractGeneratorObject;
+class DebugScriptMap;
 class PromiseObject;
 
+namespace gc {
+class AutoSuppressGC;
+}  // namespace gc
+
 /**
  * DebugAPI::onNativeCall allows the debugger to call callbacks just before
  * some native functions are to be executed. It also allows the hooks
@@ -101,22 +106,26 @@ class DebugAPI {
   // Trace all debugger-owned GC things unconditionally, during a moving GC.
   static void traceAllForMovingGC(JSTracer* trc);
 
-  // Trace debugging information for a JSScript.
-  static void traceDebugScript(JSTracer* trc, JSScript* script);
+  // Trace the debug script map.  Called as part of tracing a zone's roots.
+  static void traceDebugScriptMap(JSTracer* trc, DebugScriptMap* map);
 
   static void traceFromRealm(JSTracer* trc, Realm* realm);
 
   // The garbage collector calls this after everything has been marked, but
   // before anything has been finalized. We use this to clear Debugger /
   // debuggee edges at a point where the parties concerned are all still
-  // initialized.
-  static void sweepAll(JSFreeOp* fop);
+  // initialized. This does not update edges to moved GC things which is handled
+  // via the other trace methods.
+  static void sweepAll(JS::GCContext* gcx);
 
   // Add sweep group edges due to the presence of any debuggers.
-  static MOZ_MUST_USE bool findSweepGroupEdges(JSRuntime* rt);
+  [[nodiscard]] static bool findSweepGroupEdges(JSRuntime* rt);
 
-  // Destroy the debugging information associated with a script.
-  static void destroyDebugScript(JSFreeOp* fop, JSScript* script);
+  // Remove the debugging information associated with a script.
+  static void removeDebugScript(JS::GCContext* gcx, JSScript* script);
+
+  // Delete a Zone's debug script map. Called when a zone is destroyed.
+  static void deleteDebugScriptMap(DebugScriptMap* map);
 
   // Validate the debugging information in a script after a moving GC>
 #ifdef JSGC_HASH_TABLE_CHECKS
@@ -139,15 +148,15 @@ class DebugAPI {
 
   // Update Debugger frames when an interpreter frame is replaced with a
   // baseline frame.
-  static MOZ_MUST_USE bool handleBaselineOsr(JSContext* cx,
-                                             InterpreterFrame* from,
-                                             jit::BaselineFrame* to);
+  [[nodiscard]] static bool handleBaselineOsr(JSContext* cx,
+                                              InterpreterFrame* from,
+                                              jit::BaselineFrame* to);
 
   // Update Debugger frames when an Ion frame bails out and is replaced with a
   // baseline frame.
-  static MOZ_MUST_USE bool handleIonBailout(JSContext* cx,
-                                            jit::RematerializedFrame* from,
-                                            jit::BaselineFrame* to);
+  [[nodiscard]] static bool handleIonBailout(JSContext* cx,
+                                             jit::RematerializedFrame* from,
+                                             jit::BaselineFrame* to);
 
   // Detach any Debugger frames from an Ion frame after an error occurred while
   // it bailed out.
@@ -157,7 +166,7 @@ class DebugAPI {
   // When doing on-stack-replacement of a debuggee interpreter frame with a
   // baseline frame, ensure that the resulting frame can be observed by the
   // debugger.
-  static MOZ_MUST_USE bool ensureExecutionObservabilityOfOsrFrame(
+  [[nodiscard]] static bool ensureExecutionObservabilityOfOsrFrame(
       JSContext* cx, AbstractFramePtr osrSourceFrame);
 
   // Describes a set of scripts or frames whose execution observability can
@@ -194,8 +203,8 @@ class DebugAPI {
    * frame, |frame|. Call whatever hooks have been registered to observe new
    * frames.
    */
-  static inline MOZ_MUST_USE bool onEnterFrame(JSContext* cx,
-                                               AbstractFramePtr frame);
+  [[nodiscard]] static inline bool onEnterFrame(JSContext* cx,
+                                                AbstractFramePtr frame);
 
   /*
    * Like onEnterFrame, but for resuming execution of a generator or async
@@ -212,8 +221,8 @@ class DebugAPI {
    * JSOp::AfterYield, just after the generator resumes. The difference
    * should not be user-visible.
    */
-  static inline MOZ_MUST_USE bool onResumeFrame(JSContext* cx,
-                                                AbstractFramePtr frame);
+  [[nodiscard]] static inline bool onResumeFrame(JSContext* cx,
+                                                 AbstractFramePtr frame);
 
   static inline NativeResumeMode onNativeCall(JSContext* cx,
                                               const CallArgs& args,
@@ -227,15 +236,15 @@ class DebugAPI {
    * Note that this method is called for all |debugger;| statements,
    * regardless of the frame's debuggee-ness.
    */
-  static inline MOZ_MUST_USE bool onDebuggerStatement(JSContext* cx,
-                                                      AbstractFramePtr frame);
+  [[nodiscard]] static inline bool onDebuggerStatement(JSContext* cx,
+                                                       AbstractFramePtr frame);
 
   /*
    * Announce to the debugger that an exception has been thrown and propagated
    * to |frame|. Call whatever hooks have been registered to observe this.
    */
-  static inline MOZ_MUST_USE bool onExceptionUnwind(JSContext* cx,
-                                                    AbstractFramePtr frame);
+  [[nodiscard]] static inline bool onExceptionUnwind(JSContext* cx,
+                                                     AbstractFramePtr frame);
 
   /*
    * Announce to the debugger that the thread has exited a JavaScript frame,
@@ -252,15 +261,15 @@ class DebugAPI {
    * throw, or vice versa: we can redirect to a complete copy of the
    * alternative path, containing its own call to onLeaveFrame.)
    */
-  static inline MOZ_MUST_USE bool onLeaveFrame(JSContext* cx,
-                                               AbstractFramePtr frame,
-                                               jsbytecode* pc, bool ok);
+  [[nodiscard]] static inline bool onLeaveFrame(JSContext* cx,
+                                                AbstractFramePtr frame,
+                                                const jsbytecode* pc, bool ok);
 
   // Call any breakpoint handlers for the current scripted location.
-  static MOZ_MUST_USE bool onTrap(JSContext* cx);
+  [[nodiscard]] static bool onTrap(JSContext* cx);
 
   // Call any stepping handlers for the current scripted location.
-  static MOZ_MUST_USE bool onSingleStep(JSContext* cx);
+  [[nodiscard]] static bool onSingleStep(JSContext* cx);
 
   // Notify any Debugger instances observing this promise's global that a new
   // promise was allocated.
@@ -293,6 +302,9 @@ class DebugAPI {
   // Whether any Debugger is observing asm.js execution in a global.
   static bool debuggerObservesAsmJS(GlobalObject* global);
 
+  // Whether any Debugger is observing WebAssembly execution in a global.
+  static bool debuggerObservesWasm(GlobalObject* global);
+
   /*
    * Return true if the given global is being observed by at least one
    * Debugger that is tracking allocations.
@@ -314,8 +326,8 @@ class DebugAPI {
   /*** Assorted methods for interacting with the runtime. *********************/
 
   // Checks if the current compartment is allowed to execute code.
-  static inline MOZ_MUST_USE bool checkNoExecute(JSContext* cx,
-                                                 HandleScript script);
+  [[nodiscard]] static inline bool checkNoExecute(JSContext* cx,
+                                                  HandleScript script);
 
   /*
    * Announce to the debugger that a generator object has been created,
@@ -323,59 +335,52 @@ class DebugAPI {
    *
    * This does not fire user hooks, but it's needed for debugger bookkeeping.
    */
-  static inline MOZ_MUST_USE bool onNewGenerator(
+  [[nodiscard]] static inline bool onNewGenerator(
       JSContext* cx, AbstractFramePtr frame,
       Handle<AbstractGeneratorObject*> genObj);
 
   // If necessary, record an object that was just allocated for any observing
   // debuggers.
-  static inline MOZ_MUST_USE bool onLogAllocationSite(JSContext* cx,
-                                                      JSObject* obj,
-                                                      HandleSavedFrame frame,
-                                                      mozilla::TimeStamp when);
+  [[nodiscard]] static inline bool onLogAllocationSite(
+      JSContext* cx, JSObject* obj, Handle<SavedFrame*> frame,
+      mozilla::TimeStamp when);
 
   // Announce to the debugger that a global object is being collected by the
   // specified major GC.
   static inline void notifyParticipatesInGC(GlobalObject* global,
                                             uint64_t majorGCNumber);
 
-  /*
-   * Get any instrumentation ID which has been associated with a script using
-   * the specified debugger object.
-   */
-  static bool getScriptInstrumentationId(JSContext* cx, HandleObject dbgObject,
-                                         HandleScript script,
-                                         MutableHandleValue rval);
-
  private:
   static bool stepModeEnabledSlow(JSScript* script);
   static bool hasBreakpointsAtSlow(JSScript* script, jsbytecode* pc);
   static void slowPathOnNewGlobalObject(JSContext* cx,
                                         Handle<GlobalObject*> global);
   static void slowPathNotifyParticipatesInGC(uint64_t majorGCNumber,
-                                             JS::Realm::DebuggerVector& dbgs);
-  static MOZ_MUST_USE bool slowPathOnLogAllocationSite(
-      JSContext* cx, HandleObject obj, HandleSavedFrame frame,
-      mozilla::TimeStamp when, JS::Realm::DebuggerVector& dbgs);
-  static MOZ_MUST_USE bool slowPathOnLeaveFrame(JSContext* cx,
-                                                AbstractFramePtr frame,
-                                                jsbytecode* pc, bool ok);
-  static MOZ_MUST_USE bool slowPathOnNewGenerator(
+                                             JS::Realm::DebuggerVector& dbgs,
+                                             const JS::AutoRequireNoGC& nogc);
+  [[nodiscard]] static bool slowPathOnLogAllocationSite(
+      JSContext* cx, HandleObject obj, Handle<SavedFrame*> frame,
+      mozilla::TimeStamp when, JS::Realm::DebuggerVector& dbgs,
+      const gc::AutoSuppressGC& nogc);
+  [[nodiscard]] static bool slowPathOnLeaveFrame(JSContext* cx,
+                                                 AbstractFramePtr frame,
+                                                 const jsbytecode* pc, bool ok);
+  [[nodiscard]] static bool slowPathOnNewGenerator(
       JSContext* cx, AbstractFramePtr frame,
       Handle<AbstractGeneratorObject*> genObj);
-  static MOZ_MUST_USE bool slowPathCheckNoExecute(JSContext* cx,
-                                                  HandleScript script);
-  static MOZ_MUST_USE bool slowPathOnEnterFrame(JSContext* cx,
-                                                AbstractFramePtr frame);
-  static MOZ_MUST_USE bool slowPathOnResumeFrame(JSContext* cx,
+  [[nodiscard]] static bool slowPathCheckNoExecute(JSContext* cx,
+                                                   HandleScript script);
+  [[nodiscard]] static bool slowPathOnEnterFrame(JSContext* cx,
                                                  AbstractFramePtr frame);
+  [[nodiscard]] static bool slowPathOnResumeFrame(JSContext* cx,
+                                                  AbstractFramePtr frame);
   static NativeResumeMode slowPathOnNativeCall(JSContext* cx,
                                                const CallArgs& args,
                                                CallReason reason);
-  static MOZ_MUST_USE bool slowPathOnDebuggerStatement(JSContext* cx,
-                                                       AbstractFramePtr frame);
-  static MOZ_MUST_USE bool slowPathOnExceptionUnwind(JSContext* cx,
-                                                     AbstractFramePtr frame);
+  [[nodiscard]] static bool slowPathOnDebuggerStatement(JSContext* cx,
+                                                        AbstractFramePtr frame);
+  [[nodiscard]] static bool slowPathOnExceptionUnwind(JSContext* cx,
+                                                      AbstractFramePtr frame);
   static void slowPathOnNewWasmInstance(
       JSContext* cx, Handle<WasmInstanceObject*> wasmInstance);
   static void slowPathOnNewPromise(JSContext* cx,
