@@ -1,4 +1,4 @@
-var g = newGlobal();
+var g = newGlobal({newCompartment: true});
 var dbg = new Debugger(g);
 
 var hits = 0;
@@ -13,7 +13,7 @@ const N = 10;
 for (var i = 0; i < N; ++i) {
     g.escaped = undefined;
     g.eval("function h() { debugger }");
-    g.eval("(function () { var y = {p:42}; h(); yield })().next();");
+    g.eval("(function* () { var y = {p:42}; h(); yield })().next();");
     assertEq(g.eval("escaped().p"), 42);
     arr.push(g.escaped);
 }
