@@ -2,8 +2,8 @@
 
 load(libdir + "asserts.js");
 
-var g = newGlobal();
-g.eval("function gen(a) { debugger; yield a; }");
+var g = newGlobal({newCompartment: true});
+g.eval("function* gen(a) { debugger; yield a; }");
 g.eval("function test() { debugger; }");
 var dbg = new Debugger(g);
 var genframe;
@@ -15,5 +15,5 @@ dbg.onDebuggerStatement = function (frame) {
         assertThrowsInstanceOf(function () { genframe.eval("a"); }, Error);
     hits++;
 };
-g.eval("var it = gen(42); assertEq(it.next(), 42); test();");
+g.eval("var it = gen(42); assertEq(it.next().value, 42); test();");
 assertEq(hits, 2);
