@@ -1,7 +1,7 @@
 // Test that we only sample about allocationSamplingProbability * 100 percent of
 // allocations.
 
-const root = newGlobal();
+const root = newGlobal({newCompartment: true});
 
 const dbg = new Debugger();
 const wrappedRoot = dbg.addDebuggee(root);
@@ -32,5 +32,5 @@ dbg.memory.trackingAllocationSites = true;
 // probability is fine.
 measure(0.0, 0);
 measure(1.0, 100);
-measure(0.1, 9);
-measure(0.5, 51);
+measure(0.1, 11);
+measure(0.5, 49);
