@@ -3,7 +3,7 @@
 
 load(libdir + 'asserts.js');
 
-var g = newGlobal({ invisibleToDebugger: true });
+var g = newGlobal({ newCompartment: true, invisibleToDebugger: true });
 
 var dbg = new Debugger;
 
