@@ -8,7 +8,10 @@ dbg.addDebuggee(g);
 let globalScript;
 dbg.onNewScript = script => { globalScript = script };
 
-g.evaluate(`
+// Leave `g()` on the first line so we can check that `columnNumber` is passed to the
+// reparsed script (`columnNumber` is only used to offset breakpoint column on the first
+// line of the script).
+g.evaluate(`g();
 function f() {
   for (var i = 0; i < 10; i++) {
     g();
@@ -20,7 +23,7 @@ function g() {
 }
 
 f();
-`, { fileName: "foobar.js", lineNumber: 3 });
+`, { fileName: "foobar.js", lineNumber: 3, columnNumber: 42 });
 
 let onNewScriptCalls = 0;
 dbg.onNewScript = script => { onNewScriptCalls++; };
@@ -31,6 +34,7 @@ assertEq(onNewScriptCalls, 0);
 
 assertEq(reparsedScript.url, "foobar.js");
 assertEq(reparsedScript.startLine, 3);
+assertEq(reparsedScript.startColumn, 42);
 
 // Test for the same breakpoint positions in the original and reparsed script.
 function getBreakpointPositions(script) {
