@@ -1,7 +1,7 @@
-var g = newGlobal('new-compartment');
+var g = newGlobal({newCompartment: true});
 var dbg = new Debugger();
 var gw = dbg.addDebuggee(g);
 dbg.onDebuggerStatement = function (f) {
-	gw.evalInGlobal("eval('var x = \"A Brief History of Love\"');\n")
+	gw.executeInGlobal("eval('var x = \"A Brief History of Love\"');\n")
 };
 g.eval('debugger');
