@@ -1,5 +1,5 @@
 // |jit-test| error: ReferenceError
-var g = newGlobal();
+var g = newGlobal({newCompartment: true});
 g.parent = this;
 g.eval("new Debugger(parent).onExceptionUnwind = function () { };");
 evaluate("\
@@ -14,4 +14,4 @@ evaluate("\
             }\
         }\
     }\
-", { noScriptRval : true, compileAndGo : true });
+", { noScriptRval : true, isRunOnce: true });
