@@ -3,12 +3,12 @@
 
 /*---
 esid: sec-array.prototype.map
-description: Array.prototype.map - applied to null
+description: Array.prototype.map throws TypeError if callbackfn is undefined
 ---*/
 
-
+var arr = new Array(10);
 assert.throws(TypeError, function() {
-  Array.prototype.map.call(null); // TypeError is thrown if value is null
+  arr.map();
 });
 
 reportCompare(0, 0);
