@@ -4,24 +4,26 @@
 /*---
 esid: sec-array.prototype.reduce
 description: >
-    Array.prototype.reduce - This object is the Arguments object which
-    implements its own property get method (number of arguments is
-    less than number of parameters)
+    Array.prototype.reduce - element to be retrieved is inherited
+    accessor property without a get function on an Array
 ---*/
 
 var testResult = false;
 
 function callbackfn(prevVal, curVal, idx, obj) {
   if (idx === 1) {
-    testResult = (prevVal === 0);
+    testResult = (prevVal === undefined);
   }
 }
 
-var func = function(a, b, c) {
-  Array.prototype.reduce.call(arguments, callbackfn);
-};
+Object.defineProperty(Array.prototype, "0", {
+  set: function() {},
+  configurable: true
+});
 
-func(0, 1);
+var arr = [, 1, 2];
+
+arr.reduce(callbackfn);
 
 assert(testResult, 'testResult !== true');
 
