@@ -12,30 +12,25 @@ description: -length < start < end < 0
 var x = [0, 1, 2, 3, 4];
 var arr = x.slice(-3, -1);
 
-//CHECK#1
 arr.getClass = Object.prototype.toString;
 if (arr.getClass() !== "[object " + "Array" + "]") {
-  $ERROR('#1: var x = [0,1,2,3,4]; var arr = x.slice(-3,-1); arr is Array object. Actual: ' + (arr.getClass()));
+  throw new Test262Error('#1: var x = [0,1,2,3,4]; var arr = x.slice(-3,-1); arr is Array object. Actual: ' + (arr.getClass()));
 }
 
-//CHECK#2
 if (arr.length !== 2) {
-  $ERROR('#2: var x = [0,1,2,3,4]; var arr = x.slice(-3,-1); arr.length === 2. Actual: ' + (arr.length));
+  throw new Test262Error('#2: var x = [0,1,2,3,4]; var arr = x.slice(-3,-1); arr.length === 2. Actual: ' + (arr.length));
 }
 
-//CHECK#3
 if (arr[0] !== 2) {
-  $ERROR('#3: var x = [0,1,2,3,4]; var arr = x.slice(-3,-1); arr[0] === 2. Actual: ' + (arr[0]));
+  throw new Test262Error('#3: var x = [0,1,2,3,4]; var arr = x.slice(-3,-1); arr[0] === 2. Actual: ' + (arr[0]));
 }
 
-//CHECK#4
 if (arr[1] !== 3) {
-  $ERROR('#4: var x = [0,1,2,3,4]; var arr = x.slice(-3,-1); arr[1] === 3. Actual: ' + (arr[1]));
+  throw new Test262Error('#4: var x = [0,1,2,3,4]; var arr = x.slice(-3,-1); arr[1] === 3. Actual: ' + (arr[1]));
 }
 
-//CHECK#5
 if (arr[2] !== undefined) {
-  $ERROR('#5: var x = [0,1,2,3,4]; var arr = x.slice(-3,-1); arr[2] === undefined. Actual: ' + (arr[2]));
+  throw new Test262Error('#5: var x = [0,1,2,3,4]; var arr = x.slice(-3,-1); arr[2] === undefined. Actual: ' + (arr[2]));
 }
 
 reportCompare(0, 0);
