@@ -7,15 +7,15 @@ info: The Date property "UTC" has { DontEnum } attributes
 es5id: 15.9.4.3_A1_T3
 description: Checking DontEnum attribute
 ---*/
-
-if (Date.propertyIsEnumerable('UTC')) {
-  $ERROR('#1: The Date.UTC property has the attribute DontEnum');
-}
+assert(
+  !Date.propertyIsEnumerable('UTC'),
+  'The value of !Date.propertyIsEnumerable(\'UTC\') is expected to be true'
+);
 
 for (var x in Date) {
-  if (x === "UTC") {
-    $ERROR('#2: The Date.UTC has the attribute DontEnum');
-  }
+  assert.notSameValue(x, "UTC", 'The value of x is not "UTC"');
 }
 
+// TODO: Convert to verifyProperty() format.
+
 reportCompare(0, 0);
