@@ -6,9 +6,10 @@ info: The Date.prototype has the property "getDate"
 esid: sec-properties-of-the-date-prototype-object
 description: The Date.prototype has the property "getDate"
 ---*/
-
-if (Date.prototype.hasOwnProperty("getDate") !== true) {
-  $ERROR('#1: The Date.prototype has the property "getDate"');
-}
+assert.sameValue(
+  Date.prototype.hasOwnProperty("getDate"),
+  true,
+  'Date.prototype.hasOwnProperty("getDate") must return true'
+);
 
 reportCompare(0, 0);
