@@ -8,15 +8,15 @@ info: |
 esid: sec-date.prototype.setutcseconds
 description: Checking DontEnum attribute
 ---*/
-
-if (Date.prototype.setUTCSeconds.propertyIsEnumerable('length')) {
-  $ERROR('#1: The Date.prototype.setUTCSeconds.length property has the attribute DontEnum');
-}
+assert(
+  !Date.prototype.setUTCSeconds.propertyIsEnumerable('length'),
+  'The value of !Date.prototype.setUTCSeconds.propertyIsEnumerable(\'length\') is expected to be true'
+);
 
 for (var x in Date.prototype.setUTCSeconds) {
-  if (x === "length") {
-    $ERROR('#2: The Date.prototype.setUTCSeconds.length has the attribute DontEnum');
-  }
+  assert.notSameValue(x, "length", 'The value of x is not "length"');
 }
 
+// TODO: Convert to verifyProperty() format.
+
 reportCompare(0, 0);
