@@ -8,12 +8,18 @@ description: Checking absence of ReadOnly attribute
 ---*/
 
 var x = Date.prototype.toLocaleString;
-if (x === 1)
+if (x === 1) {
   Date.prototype.toLocaleString = 2;
-else
+} else {
   Date.prototype.toLocaleString = 1;
-if (Date.prototype.toLocaleString === x) {
-  $ERROR('#1: The Date.prototype.toLocaleString has not the attribute ReadOnly');
 }
 
+assert.notSameValue(
+  Date.prototype.toLocaleString,
+  x,
+  'The value of Date.prototype.toLocaleString is expected to not equal the value of `x`'
+);
+
+// TODO: Convert to verifyProperty() format.
+
 reportCompare(0, 0);
