@@ -6,9 +6,6 @@ info: The Function constructor has the property "prototype"
 es5id: 15.3.3_A1
 description: Checking existence of the property "prototype"
 ---*/
-
-if (!Function.hasOwnProperty("prototype")) {
-  $ERROR('#1: The Function constructor has the property "prototype"');
-}
+assert(Function.hasOwnProperty("prototype"), 'Function.hasOwnProperty("prototype") must return true');
 
 reportCompare(0, 0);
