@@ -2,12 +2,12 @@
 // This code is governed by the BSD license found in the LICENSE file.
 
 /*---
-es5id: 15.2.3.2-2-16
+es5id: 15.2.3.2-2-12
 description: >
     Object.getPrototypeOf returns the [[Prototype]] of its parameter
-    (TypeError)
+    (EvalError)
 ---*/
 
-assert.sameValue(Object.getPrototypeOf(TypeError), Error, 'Object.getPrototypeOf(TypeError)');
+assert.sameValue(Object.getPrototypeOf(EvalError), Error, 'Object.getPrototypeOf(EvalError)');
 
 reportCompare(0, 0);
