@@ -3,20 +3,18 @@
 
 /*---
 info: |
-    The | regular expression operator separates two alternatives.
-    The pattern first tries to match the left Alternative (followed by the sequel of the regular expression).
-    If it fails, it tries to match the right Disjunction (followed by the sequel of the regular expression)
-es5id: 15.10.2.3_A1_T4
-description: >
-    Execute /\d{3}|[a-z]{4}/.exec("2, 12 and 234 AND of course repeat
-    12") and check results
+    The production CharacterClass :: [ [lookahead \notin {^}] ClassRanges ]
+    evaluates by evaluating ClassRanges to obtain a CharSet and returning
+    that CharSet and the boolean false
+es5id: 15.10.2.13_A1_T4
+description: Execute /q[ax-zb](?=\s+)/.exec("tqaqy ") and check results
 ---*/
 
-var __executed = /\d{3}|[a-z]{4}/.exec("2, 12 and 234 AND of course repeat 12");
+var __executed = /q[ax-zb](?=\s+)/.exec("tqaqy ");
 
-var __expected = ["234"];
-__expected.index = 10;
-__expected.input = "2, 12 and 234 AND of course repeat 12";
+var __expected = ["qy"];
+__expected.index = 3;
+__expected.input = "tqaqy ";
 
 assert.sameValue(
   __executed.length,
