@@ -2,19 +2,16 @@
 // This code is governed by the BSD license found in the LICENSE file.
 
 /*---
-info: |
-    The production QuantifierPrefix :: { DecimalDigits , }evaluates as follows:
-    i) Let i be the MV of DecimalDigits
-    ii) Return the two results i and \infty
-es5id: 15.10.2.7_A6_T4
-description: Execute /(123){1,}/.exec("123123") and check results
+info: Assertions in combination
+es5id: 15.10.2.6_A6_T4
+description: "Execute /^.*(:|$)/.exec(\"Hello: World\") and check results"
 ---*/
 
-var __executed = /(123){1,}/.exec("123123");
+var __executed = /^.*(:|$)/.exec("Hello: World");
 
-var __expected = ["123123","123"];
+var __expected = ["Hello: World", ""];
 __expected.index = 0;
-__expected.input = "123123";
+__expected.input = "Hello: World";
 
 assert.sameValue(
   __executed.length,
