@@ -11,11 +11,10 @@ info: |
     a. Let numericIndex be ! CanonicalNumericIndexString(P).
     b. If numericIndex is not undefined, then
       ...
-      vi. If numericIndex ≥ the value of O's [[ArrayLength]] internal slot,
-      return false.
+      iii. If ! IsValidIntegerIndex(O, numericIndex) is false, return false.
   ...
 includes: [testBigIntTypedArray.js]
-features: [BigInt, Reflect, TypedArray]
+features: [align-detached-buffer-semantics-with-web-reality, BigInt, Reflect, TypedArray]
 ---*/
 
 // Prevents false positives using OrdinaryHasProperty
@@ -24,7 +23,7 @@ TypedArray.prototype[1] = "test262";
 testWithBigIntTypedArrayConstructors(function(TA) {
   var sample = new TA(1);
 
-  assert.sameValue(Reflect.has(sample, "1"), false, "1");
+  assert.sameValue(Reflect.has(sample, "1"), false, 'Reflect.has(sample, "1") must return false');
 });
 
 reportCompare(0, 0);
