@@ -3,14 +3,10 @@
 
 /*---
 info: |
-    If B = 11110xxx (n = 4) and (string.charAt(k + 4) and
-    string.charAt(k + 5)) or (string.charAt(k + 7) and
-    string.charAt(k + 8)) or (string.charAt(k + 10) and
-    string.charAt(k + 11)) do not represent hexadecimal digits, throw URIError
+    If B = 110xxxxx (n = 2) and string.charAt(k + 4) and
+    string.charAt(k + 5) do not represent hexadecimal digits, throw URIError
 esid: sec-decodeuricomponent-encodeduricomponent
-description: >
-    Complex tests, string.charAt(k + 4) and string.charAt(k + 5)  do
-    not represent hexadecimal digits
+description: Complex tests
 ---*/
 
 //CHECK
@@ -24,7 +20,7 @@ var interval = [
 for (var indexI = 0; indexI < interval.length; indexI++) {
   for (var indexJ = interval[indexI][0]; indexJ <= interval[indexI][1]; indexJ++) {
     try {
-      decodeURIComponent("%F0%" + String.fromCharCode(indexJ, indexJ) + "%A0%A0");
+      decodeURIComponent("%C0%" + String.fromCharCode(indexJ, indexJ));
       result = false;
     } catch (e) {
       if ((e instanceof URIError) !== true) {
@@ -35,7 +31,7 @@ for (var indexI = 0; indexI < interval.length; indexI++) {
 }
 
 if (result !== true) {
-  throw new Test262Error('#1: If B = 11110xxx (n = 4) and (string.charAt(k + 4) and string.charAt(k + 5)) do not represent hexadecimal digits, throw URIError');
+  throw new Test262Error('#1: If B = 110xxxxx (n = 2) and (string.charAt(k + 4) and  string.charAt(k + 5)) do not represent hexadecimal digits, throw URIError');
 }
 
 reportCompare(0, 0);
