@@ -11,11 +11,10 @@ info: |
 includes: [compareArray.js]
 ---*/
 
-assert(compareArray(
-  Intl.getCanonicalLocales(
-    ['ab-cd', 'ff', 'de-rt', 'ab-Cd']), ['ab-CD', 'ff', 'de-RT']));
+assert.compareArray(Intl.getCanonicalLocales(
+  ['ab-cd', 'ff', 'de-rt', 'ab-Cd']), ['ab-CD', 'ff', 'de-RT']);
 
 var locales = Intl.getCanonicalLocales(['en-US', 'en-US']);
-assert(compareArray(locales, ['en-US']), 'en-US');
+assert.compareArray(locales, ['en-US'], 'en-US');
 
 reportCompare(0, 0);
