@@ -7,7 +7,7 @@ description: >
     The left-hand side expression is evaluated before the right-hand side.
     Left-hand side expression is MemberExpression: base[prop]. ToPropertyKey(prop)
     is only called once.
-    Check operator is "x ^= y".
+    Check operator is "x <<= y".
 ---*/
 
 var propKeyEvaluated = false;
@@ -23,6 +23,6 @@ var expr = function() {
   return 0;
 };
 
-base[prop] ^= expr();
+base[prop] <<= expr();
 
 reportCompare(0, 0);
