@@ -10,13 +10,13 @@ description: Checking with "="
 //CHECK#1 
 var OBJECT = 0;
 if ((OBJECT = Object, {}) instanceof OBJECT !== true) {
-  $ERROR('#1: var OBJECT = 0; (OBJECT = Object, {}) instanceof OBJECT === true');
+  throw new Test262Error('#1: var OBJECT = 0; (OBJECT = Object, {}) instanceof OBJECT === true');
 }
 
 //CHECK#2
 var object = {}; 
 if (object instanceof (object = 0, Object) !== true) {
-  $ERROR('#2: var object = {};  object instanceof (object = 0, Object) === true');
+  throw new Test262Error('#2: var object = {};  object instanceof (object = 0, Object) === true');
 }
 
 reportCompare(0, 0);
