@@ -12,13 +12,13 @@ description: Create function. It have property, that returned "this"
 //CHECK#1
 function SetThis() {this.THIS = this}
 if ((new SetThis()).THIS.toString() !== "[object Object]") {
-  $ERROR('#1: function SetThis() {this.THIS = this} (new SetThis()).THIS.toString() !== "[object Object]". Actual: ' + ((new SetThis()).THIS.toString()));
+  throw new Test262Error('#1: function SetThis() {this.THIS = this} (new SetThis()).THIS.toString() !== "[object Object]". Actual: ' + ((new SetThis()).THIS.toString()));
 }
 
 //CHECK#2
 function SetEvalThis() {this.THIS = eval("this")}
 if ((new SetEvalThis()).THIS.toString() !== "[object Object]") {
-  $ERROR('#2: function SetEvalThis() {this.THIS = eval("this")} (new SetEvalThis()).THIS.toString() !== "[object Object]". Actual: ' + ((new SetEvalThis()).THIS.toString()));
+  throw new Test262Error('#2: function SetEvalThis() {this.THIS = eval("this")} (new SetEvalThis()).THIS.toString() !== "[object Object]". Actual: ' + ((new SetEvalThis()).THIS.toString()));
 }
 
 reportCompare(0, 0);
