@@ -11,16 +11,16 @@ description: False and true convert to Number by implicit transformation
 
 // CHECK#1
 if (+(false) !== +0) {
-  $ERROR('#1.1: +(false) === 0. Actual: ' + (+(false)));
+  throw new Test262Error('#1.1: +(false) === 0. Actual: ' + (+(false)));
 } else {
   if (1/+(false) !== Number.POSITIVE_INFINITY) {
-    $ERROR('#1.2: +(false) === +0. Actual: -0');
+    throw new Test262Error('#1.2: +(false) === +0. Actual: -0');
   }
 }
 
 // CHECK#2
 if (+(true) !== 1) {
-  $ERROR('#2: +(true) === 1. Actual: ' + (+(true)));	
+  throw new Test262Error('#2: +(true) === 1. Actual: ' + (+(true)));	
 }
 
 reportCompare(0, 0);
