@@ -3,20 +3,17 @@
 // This code is governed by the BSD license found in the LICENSE file.
 
 /*---
-es5id: 10.4.3-1-31gs
+es5id: 10.4.3-1-9gs
 description: >
     Strict - checking 'this' from a global scope (FunctionExpression
-    defined within a FunctionExpression inside strict mode)
+    defined within strict mode)
 flags: [onlyStrict]
 ---*/
 
-var f1 = function () {
-    var f = function () {
-        return typeof this;
-    }
-    return (f()==="undefined") && ((typeof this)==="undefined");
+var f = function () {
+    return typeof this;
 }
-if (! f1()) {
+if (f() !== "undefined") {
     throw "'this' had incorrect value!";
 }
 
