@@ -3,17 +3,26 @@
 // This code is governed by the BSD license found in the LICENSE file.
 
 /*---
-esid: prod-RegularExpressionChar
+esid: sec-literals-regular-expression-literals
 info: |
   RegularExpressionChar ::
-    RegularExpressionNonTerminator but not one of \ or / or [
+    RegularExpressionBackslashSequence
+
+
+  RegularExpressionBackslashSequence ::
+    \ RegularExpressionNonTerminator
 
   RegularExpressionNonTerminator ::
     SourceCharacter but not LineTerminator
 
-description: >
-  A regular express may not contain a "\" as a SourceCharacter
+  LineTerminator ::
+    <LF>
+    <CR>
+    <LS>
+    <PS>
 
+description: >
+  A regular expression may not contain a <LF> as a SourceCharacter
 negative:
   phase: parse
   type: SyntaxError
@@ -21,4 +30,5 @@ negative:
 
 $DONOTEVALUATE();
 
-/a\/
+/a
+/
