@@ -20,7 +20,7 @@ function __func(){
 //////////////////////////////////////////////////////////////////////////////
 //CHECK#1
 if (typeof __func("A","B",1,2) !== "object") {
-	$ERROR('#1: arguments property has attribute { DontDelete }');
+	throw new Test262Error('#1: arguments property has attribute { DontDelete }');
 }
 //
 //////////////////////////////////////////////////////////////////////////////
