@@ -12,7 +12,7 @@ try{
   throw "exception"+" #1";
 }
 catch(e){
-  if (e!=="exception #1") $ERROR('#1: Exception === "exception #1"(operaton +). Actual:  Exception ==='+ e );
+  if (e!=="exception #1") throw new Test262Error('#1: Exception === "exception #1"(operaton +). Actual:  Exception ==='+ e );
 }
 
 // CHECK#2
@@ -22,7 +22,7 @@ try{
   throw b+a;
 }
 catch(e){
-  if (e!=="exception #1") $ERROR('#2: Exception === "exception #1"(operaton +). Actual:  Exception ==='+ e );
+  if (e!=="exception #1") throw new Test262Error('#2: Exception === "exception #1"(operaton +). Actual:  Exception ==='+ e );
 }
 
 reportCompare(0, 0);
