@@ -12,20 +12,15 @@
 
 #define __STDC_FORMAT_MACROS
 
-#include "mozilla/Attributes.h"
 #include "mozilla/Maybe.h"
 #include "mozilla/ReverseIterator.h"
 #include "mozilla/Sprintf.h"
-#include "mozilla/Vector.h"
 
-#include <algorithm>
 #include <inttypes.h>
 #include <stdio.h>
 #include <string.h>
-#include <type_traits>
 
 #include "jsapi.h"
-#include "jsnum.h"
 #include "jstypes.h"
 
 #include "frontend/BytecodeCompiler.h"
@@ -33,11 +28,18 @@
 #include "gc/PublicIterators.h"
 #include "jit/IonScript.h"  // IonBlockCounts
 #include "js/CharacterEncoding.h"
+#include "js/experimental/CodeCoverage.h"
+#include "js/experimental/PCCountProfiling.h"  // JS::{Start,Stop}PCCountProfiling, JS::PurgePCCounts, JS::GetPCCountScript{Count,Summary,Contents}
+#include "js/friend/DumpFunctions.h"           // js::DumpPC, js::DumpScript
+#include "js/friend/ErrorMessages.h"           // js::GetErrorMessage, JSMSG_*
+#include "js/Printer.h"
 #include "js/Printf.h"
 #include "js/Symbol.h"
+#include "util/DifferentialTesting.h"
 #include "util/Memory.h"
-#include "util/StringBuffer.h"
 #include "util/Text.h"
+#include "vm/BuiltinObjectKind.h"
+#include "vm/BytecodeIterator.h"  // for AllBytecodesIterable
 #include "vm/BytecodeLocation.h"
 #include "vm/CodeCoverage.h"
 #include "vm/EnvironmentObject.h"
@@ -46,17 +48,17 @@
 #include "vm/JSContext.h"
 #include "vm/JSFunction.h"
 #include "vm/JSObject.h"
+#include "vm/JSONPrinter.h"
 #include "vm/JSScript.h"
 #include "vm/Opcodes.h"
 #include "vm/Realm.h"
 #include "vm/Shape.h"
-#include "vm/ToSource.h"  // js::ValueToSource
+#include "vm/ToSource.h"       // js::ValueToSource
+#include "vm/WellKnownAtom.h"  // js_*_str
 
 #include "gc/GC-inl.h"
 #include "vm/BytecodeIterator-inl.h"
-#include "vm/BytecodeLocation-inl.h"
 #include "vm/JSContext-inl.h"
-#include "vm/JSObject-inl.h"
 #include "vm/JSScript-inl.h"
 #include "vm/Realm-inl.h"
 
@@ -103,8 +105,8 @@ static bool DecompileArgumentFromStack(JSContext* cx, int formalIndex,
 
 /* static */ const char PCCounts::numExecName[] = "interp";
 
-static MOZ_MUST_USE bool DumpIonScriptCounts(Sprinter* sp, HandleScript script,
-                                             jit::IonScriptCounts* ionCounts) {
+[[nodiscard]] static bool DumpIonScriptCounts(Sprinter* sp, HandleScript script,
+                                              jit::IonScriptCounts* ionCounts) {
   if (!sp->jsprintf("IonScript [%zu blocks]:\n", ionCounts->numBlocks())) {
     return false;
   }
@@ -139,8 +141,8 @@ static MOZ_MUST_USE bool DumpIonScriptCounts(Sprinter* sp, HandleScript script,
   return true;
 }
 
-static MOZ_MUST_USE bool DumpPCCounts(JSContext* cx, HandleScript script,
-                                      Sprinter* sp) {
+[[nodiscard]] static bool DumpPCCounts(JSContext* cx, HandleScript script,
+                                       Sprinter* sp) {
   MOZ_ASSERT(script->hasScriptCounts());
 
   // Ensure the Disassemble1 call below does not discard the script counts.
@@ -207,14 +209,16 @@ bool js::DumpRealmPCCounts(JSContext* cx) {
       return false;
     }
 
-    fprintf(stdout, "--- SCRIPT %s:%u ---\n", script->filename(),
-            script->lineno());
+    const char* filename = script->filename();
+    if (!filename) {
+      filename = "(unknown)";
+    }
+    fprintf(stdout, "--- SCRIPT %s:%u ---\n", filename, script->lineno());
     if (!DumpPCCounts(cx, script, &sprinter)) {
       return false;
     }
     fputs(sprinter.string(), stdout);
-    fprintf(stdout, "--- END SCRIPT %s:%u ---\n", script->filename(),
-            script->lineno());
+    fprintf(stdout, "--- END SCRIPT %s:%u ---\n", filename, script->lineno());
   }
 
   return true;
@@ -554,9 +558,9 @@ uint32_t BytecodeParser::simulateOp(JSOp op, uint32_t offset,
   uint32_t nuses = GetUseCount(pc);
   uint32_t ndefs = GetDefCount(pc);
 
-  MOZ_ASSERT(stackDepth >= nuses);
+  MOZ_RELEASE_ASSERT(stackDepth >= nuses);
   stackDepth -= nuses;
-  MOZ_ASSERT(stackDepth + ndefs <= maximumStackDepth());
+  MOZ_RELEASE_ASSERT(stackDepth + ndefs <= maximumStackDepth());
 
 #ifdef DEBUG
   if (isStackDump) {
@@ -585,6 +589,7 @@ uint32_t BytecodeParser::simulateOp(JSOp op, uint32_t offset,
       case JSOp::InitHiddenElem:
       case JSOp::InitHiddenElemGetter:
       case JSOp::InitHiddenElemSetter:
+      case JSOp::InitLockedElem:
         // Keep the third value.
         MOZ_ASSERT(nuses == 3);
         MOZ_ASSERT(ndefs == 1);
@@ -683,7 +688,6 @@ uint32_t BytecodeParser::simulateOp(JSOp op, uint32_t offset,
     case JSOp::SetIntrinsic:
     case JSOp::SetLocal:
     case JSOp::InitAliasedLexical:
-    case JSOp::IterNext:
     case JSOp::CheckLexical:
     case JSOp::CheckAliasedLexical:
       // Keep the top value.
@@ -733,13 +737,20 @@ uint32_t BytecodeParser::simulateOp(JSOp op, uint32_t offset,
 
     case JSOp::IsGenClosing:
     case JSOp::IsNoIter:
+    case JSOp::IsNullOrUndefined:
     case JSOp::MoreIter:
-    case JSOp::OptimizeSpreadCall:
       // Keep the top value and push one more value.
       MOZ_ASSERT(nuses == 1);
       MOZ_ASSERT(ndefs == 2);
       offsetStack[stackDepth + 1].set(offset, 1);
       break;
+
+    case JSOp::CheckPrivateField:
+      // Keep the top two values, and push one new value.
+      MOZ_ASSERT(nuses == 2);
+      MOZ_ASSERT(ndefs == 3);
+      offsetStack[stackDepth + 2].set(offset, 2);
+      break;
   }
 
 #ifdef DEBUG
@@ -753,7 +764,8 @@ end:
 bool BytecodeParser::recordBytecode(uint32_t offset,
                                     const OffsetAndDefIndex* offsetStack,
                                     uint32_t stackDepth) {
-  MOZ_ASSERT(offset < script_->length());
+  MOZ_RELEASE_ASSERT(offset < script_->length());
+  MOZ_RELEASE_ASSERT(stackDepth <= maximumStackDepth());
 
   Bytecode*& code = codeArray_[offset];
   if (!code) {
@@ -831,7 +843,7 @@ bool BytecodeParser::parse() {
     // Next bytecode to analyze.
     nextOffset = offset + GetBytecodeLength(pc);
 
-    MOZ_ASSERT(*pc < JSOP_LIMIT);
+    MOZ_RELEASE_ASSERT(*pc < JSOP_LIMIT);
     JSOp op = JSOp(*pc);
 
     if (!code) {
@@ -857,14 +869,14 @@ bool BytecodeParser::parse() {
 
     uint32_t stackDepth = simulateOp(op, offset, offsetStack, code->stackDepth);
 
-#ifdef DEBUG
+#if defined(DEBUG) || defined(JS_JITSPEW)
     if (isStackDump) {
       if (!code->captureOffsetStackAfter(alloc(), offsetStack, stackDepth)) {
         reportOOM();
         return false;
       }
     }
-#endif /* DEBUG */
+#endif /* defined(DEBUG) || defined(JS_JITSPEW) */
 
     switch (op) {
       case JSOp::TableSwitch: {
@@ -909,7 +921,13 @@ bool BytecodeParser::parse() {
                 return false;
               }
             } else if (tn.kind() == TryNoteKind::Finally) {
-              if (!addJump(catchOffset, stackDepth, offsetStack, pc,
+              // Two additional values will be on the stack at the beginning
+              // of the finally block: the exception/resume index, and the
+              // |throwing| value. For the benefit of the decompiler, point
+              // them at this Try.
+              offsetStack[stackDepth].set(offset, 0);
+              offsetStack[stackDepth + 1].set(offset, 1);
+              if (!addJump(catchOffset, stackDepth + 2, offsetStack, pc,
                            JumpKind::TryFinally)) {
                 return false;
               }
@@ -977,7 +995,7 @@ static unsigned Disassemble1(JSContext* cx, HandleScript script, jsbytecode* pc,
  * current line. If showAll is true, include the source note type and the
  * entry stack depth.
  */
-static MOZ_MUST_USE bool DisassembleAtPC(
+[[nodiscard]] static bool DisassembleAtPC(
     JSContext* cx, JSScript* scriptArg, bool lines, const jsbytecode* pc,
     bool showAll, Sprinter* sp,
     DisassembleSkeptically skeptically = DisassembleSkeptically::No) {
@@ -1108,7 +1126,7 @@ bool js::Disassemble(JSContext* cx, HandleScript script, bool lines,
   return DisassembleAtPC(cx, script, lines, nullptr, false, sp, skeptically);
 }
 
-JS_FRIEND_API bool js::DumpPC(JSContext* cx, FILE* fp) {
+JS_PUBLIC_API bool js::DumpPC(JSContext* cx, FILE* fp) {
   gc::AutoSuppressGC suppressGC(cx);
   Sprinter sprinter(cx);
   if (!sprinter.init()) {
@@ -1125,7 +1143,7 @@ JS_FRIEND_API bool js::DumpPC(JSContext* cx, FILE* fp) {
   return ok;
 }
 
-JS_FRIEND_API bool js::DumpScript(JSContext* cx, JSScript* scriptArg,
+JS_PUBLIC_API bool js::DumpScript(JSContext* cx, JSScript* scriptArg,
                                   FILE* fp) {
   gc::AutoSuppressGC suppressGC(cx);
   Sprinter sprinter(cx);
@@ -1160,7 +1178,8 @@ static UniqueChars ToDisassemblySource(JSContext* cx, HandleValue v) {
     }
 
     if (obj.is<RegExpObject>()) {
-      JSString* source = obj.as<RegExpObject>().toString(cx);
+      Rooted<RegExpObject*> reobj(cx, &obj.as<RegExpObject>());
+      JSString* source = RegExpObject::toString(cx, reobj);
       if (!source) {
         return nullptr;
       }
@@ -1175,7 +1194,7 @@ static UniqueChars ToDisassemblySource(JSContext* cx, HandleValue v) {
   return QuoteString(cx, str);
 }
 
-static bool ToDisassemblySource(JSContext* cx, HandleScope scope,
+static bool ToDisassemblySource(JSContext* cx, Handle<Scope*> scope,
                                 UniqueChars* bytes) {
   UniqueChars source = JS_smprintf("%s {", ScopeKindString(scope->kind()));
   if (!source) {
@@ -1317,6 +1336,42 @@ static bool DecompileAtPCForStackDump(
     JSContext* cx, HandleScript script,
     const OffsetAndDefIndex& offsetAndDefIndex, Sprinter* sp);
 
+static bool PrintShapeProperties(JSContext* cx, Sprinter* sp,
+                                 SharedShape* shape) {
+  // Add all property keys to a vector to allow printing them in property
+  // definition order.
+  Vector<PropertyKey> props(cx);
+  for (SharedShapePropertyIter<NoGC> iter(shape); !iter.done(); iter++) {
+    if (!props.append(iter->key())) {
+      return false;
+    }
+  }
+
+  if (!sp->put("{")) {
+    return false;
+  }
+
+  for (size_t i = props.length(); i > 0; i--) {
+    PropertyKey key = props[i - 1];
+    RootedValue keyv(cx, IdToValue(key));
+    JSString* str = ToString<NoGC>(cx, keyv);
+    if (!str) {
+      ReportOutOfMemory(cx);
+      return false;
+    }
+    if (!sp->putString(str)) {
+      return false;
+    }
+    if (i > 1) {
+      if (!sp->put(", ")) {
+        return false;
+      }
+    }
+  }
+
+  return sp->put("}");
+}
+
 static unsigned Disassemble1(JSContext* cx, HandleScript script, jsbytecode* pc,
                              unsigned loc, bool lines,
                              const BytecodeParser* parser, Sprinter* sp) {
@@ -1415,16 +1470,8 @@ static unsigned Disassemble1(JSContext* cx, HandleScript script, jsbytecode* pc,
       break;
     }
 
-    case JOF_CODE_OFFSET: {
-      ptrdiff_t off = GET_CODE_OFFSET(pc);
-      if (!sp->jsprintf(" %u (%+d)", unsigned(loc + int(off)), int(off))) {
-        return 0;
-      }
-      break;
-    }
-
     case JOF_SCOPE: {
-      RootedScope scope(cx, script->getScope(GET_UINT32_INDEX(pc)));
+      Rooted<Scope*> scope(cx, script->getScope(pc));
       UniqueChars bytes;
       if (!ToDisassemblySource(cx, scope, &bytes)) {
         return 0;
@@ -1448,9 +1495,26 @@ static unsigned Disassemble1(JSContext* cx, HandleScript script, jsbytecode* pc,
       }
       break;
     }
-
+    case JOF_DEBUGCOORD: {
+      EnvironmentCoordinate ec(pc);
+      if (!sp->jsprintf("(hops = %u, slot = %u)", ec.hops(), ec.slot())) {
+        return 0;
+      }
+      break;
+    }
     case JOF_ATOM: {
-      RootedValue v(cx, StringValue(script->getAtom(GET_UINT32_INDEX(pc))));
+      RootedValue v(cx, StringValue(script->getAtom(pc)));
+      UniqueChars bytes = ToDisassemblySource(cx, v);
+      if (!bytes) {
+        return 0;
+      }
+      if (!sp->jsprintf(" %s", bytes.get())) {
+        return 0;
+      }
+      break;
+    }
+    case JOF_STRING: {
+      RootedValue v(cx, StringValue(script->getString(pc)));
       UniqueChars bytes = ToDisassemblySource(cx, v);
       if (!bytes) {
         return 0;
@@ -1482,15 +1546,7 @@ static unsigned Disassemble1(JSContext* cx, HandleScript script, jsbytecode* pc,
     }
 
     case JOF_OBJECT: {
-      /* Don't call obj.toSource if analysis/inference is active. */
-      if (script->zone()->types.activeAnalysis) {
-        if (!sp->jsprintf(" object")) {
-          return 0;
-        }
-        break;
-      }
-
-      JSObject* obj = script->getObject(GET_UINT32_INDEX(pc));
+      JSObject* obj = script->getObject(pc);
       {
         RootedValue v(cx, ObjectValue(*obj));
         UniqueChars bytes = ToDisassemblySource(cx, v);
@@ -1504,6 +1560,17 @@ static unsigned Disassemble1(JSContext* cx, HandleScript script, jsbytecode* pc,
       break;
     }
 
+    case JOF_SHAPE: {
+      SharedShape* shape = script->getShape(pc);
+      if (!sp->put(" ")) {
+        return 0;
+      }
+      if (!PrintShapeProperties(cx, sp, shape)) {
+        return 0;
+      }
+      break;
+    }
+
     case JOF_REGEXP: {
       js::RegExpObject* obj = script->getRegExp(pc);
       RootedValue v(cx, ObjectValue(*obj));
@@ -1558,6 +1625,12 @@ static unsigned Disassemble1(JSContext* cx, HandleScript script, jsbytecode* pc,
       }
       break;
 
+    case JOF_GCTHING:
+      if (!sp->jsprintf(" %u", unsigned(GET_GCTHING_INDEX(pc)))) {
+        return 0;
+      }
+      break;
+
     case JOF_UINT32:
       if (!sp->jsprintf(" %u", GET_UINT32(pc))) {
         return 0;
@@ -1577,18 +1650,14 @@ static unsigned Disassemble1(JSContext* cx, HandleScript script, jsbytecode* pc,
       }
       break;
 
-    case JOF_CLASS_CTOR: {
-      uint32_t atomIndex = 0;
-      uint32_t classStartOffset = 0, classEndOffset = 0;
-      GetClassConstructorOperands(pc, &atomIndex, &classStartOffset,
-                                  &classEndOffset);
-      RootedValue v(cx, StringValue(script->getAtom(atomIndex)));
-      UniqueChars bytes = ToDisassemblySource(cx, v);
-      if (!bytes) {
+    case JOF_TWO_UINT8: {
+      int one = (int)GET_UINT8(pc);
+      int two = (int)GET_UINT8(pc + 1);
+
+      if (!sp->jsprintf(" %d", one)) {
         return 0;
       }
-      if (!sp->jsprintf(" %s (off: %u-%u)", bytes.get(), classStartOffset,
-                        classEndOffset)) {
+      if (!sp->jsprintf(" %d", two)) {
         return 0;
       }
       break;
@@ -1712,6 +1781,7 @@ struct ExpressionDecompiler {
   bool decompilePC(const OffsetAndDefIndex& offsetAndDefIndex);
   JSAtom* getArg(unsigned slot);
   JSAtom* loadAtom(jsbytecode* pc);
+  JSString* loadString(jsbytecode* pc);
   bool quote(JSString* s, char quote);
   bool write(const char* s);
   bool write(JSString* str);
@@ -1783,6 +1853,10 @@ bool ExpressionDecompiler::decompilePC(jsbytecode* pc, uint8_t defIndex) {
         if (result) {
           return write(result.get());
         }
+
+        // If it fails, do not return parameter name and let the caller
+        // fallback.
+        return write("(intermediate value)");
       }
 
       JSAtom* atom = getArg(slot);
@@ -1804,13 +1878,10 @@ bool ExpressionDecompiler::decompilePC(jsbytecode* pc, uint8_t defIndex) {
 
     case JSOp::DelProp:
     case JSOp::StrictDelProp:
-    case JSOp::Length:
     case JSOp::GetProp:
-    case JSOp::GetBoundName:
-    case JSOp::CallProp: {
+    case JSOp::GetBoundName: {
       bool hasDelete = op == JSOp::DelProp || op == JSOp::StrictDelProp;
-      RootedAtom prop(cx,
-                      (op == JSOp::Length) ? cx->names().length : loadAtom(pc));
+      Rooted<JSAtom*> prop(cx, loadAtom(pc));
       MOZ_ASSERT(prop);
       return (hasDelete ? write("(delete ") : true) &&
              decompilePCForStackOperand(pc, -1) &&
@@ -1820,7 +1891,7 @@ bool ExpressionDecompiler::decompilePC(jsbytecode* pc, uint8_t defIndex) {
              (hasDelete ? write(")") : true);
     }
     case JSOp::GetPropSuper: {
-      RootedAtom prop(cx, loadAtom(pc));
+      Rooted<JSAtom*> prop(cx, loadAtom(pc));
       return write("super.") && quote(prop, '\0');
     }
     case JSOp::SetElem:
@@ -1834,8 +1905,7 @@ bool ExpressionDecompiler::decompilePC(jsbytecode* pc, uint8_t defIndex) {
 
     case JSOp::DelElem:
     case JSOp::StrictDelElem:
-    case JSOp::GetElem:
-    case JSOp::CallElem: {
+    case JSOp::GetElem: {
       bool hasDelete = (op == JSOp::DelElem || op == JSOp::StrictDelElem);
       return (hasDelete ? write("(delete ") : true) &&
              decompilePCForStackOperand(pc, -2) && write("[") &&
@@ -1860,7 +1930,7 @@ bool ExpressionDecompiler::decompilePC(jsbytecode* pc, uint8_t defIndex) {
     case JSOp::Int32:
       return sprinter.printf("%d", GetBytecodeInteger(pc));
     case JSOp::String:
-      return quote(loadAtom(pc), '"');
+      return quote(loadString(pc), '"');
     case JSOp::Symbol: {
       unsigned i = uint8_t(pc[1]);
       MOZ_ASSERT(i < JS::WellKnownSymbolLimit);
@@ -1872,16 +1942,17 @@ bool ExpressionDecompiler::decompilePC(jsbytecode* pc, uint8_t defIndex) {
     case JSOp::Undefined:
       return write(js_undefined_str);
     case JSOp::GlobalThis:
+    case JSOp::NonSyntacticGlobalThis:
       // |this| could convert to a very long object initialiser, so cite it by
       // its keyword name.
       return write(js_this_str);
     case JSOp::NewTarget:
       return write("new.target");
     case JSOp::Call:
+    case JSOp::CallContent:
     case JSOp::CallIgnoresRv:
     case JSOp::CallIter:
-    case JSOp::FunCall:
-    case JSOp::FunApply: {
+    case JSOp::CallContentIter: {
       uint16_t argc = GET_ARGC(pc);
       return decompilePCForStackOperand(pc, -int32_t(argc + 2)) &&
              write(argc ? "(...)" : "()");
@@ -1891,36 +1962,15 @@ bool ExpressionDecompiler::decompilePC(jsbytecode* pc, uint8_t defIndex) {
     case JSOp::NewArray:
       return write("[]");
     case JSOp::RegExp: {
-      RootedObject obj(cx, script->getObject(GET_UINT32_INDEX(pc)));
-      JSString* str = obj->as<RegExpObject>().toString(cx);
+      Rooted<RegExpObject*> obj(cx, &script->getObject(pc)->as<RegExpObject>());
+      JSString* str = RegExpObject::toString(cx, obj);
       if (!str) {
         return false;
       }
       return write(str);
     }
-    case JSOp::NewArrayCopyOnWrite: {
-      RootedObject obj(cx, script->getObject(GET_UINT32_INDEX(pc)));
-      Handle<ArrayObject*> aobj = obj.as<ArrayObject>();
-      if (!write("[")) {
-        return false;
-      }
-      for (size_t i = 0; i < aobj->getDenseInitializedLength(); i++) {
-        if (i > 0 && !write(", ")) {
-          return false;
-        }
-
-        RootedValue v(cx, aobj->getDenseElement(i));
-        MOZ_RELEASE_ASSERT(v.isPrimitive() && !v.isMagic());
-
-        JSString* str = ValueToSource(cx, v);
-        if (!str || !write(str)) {
-          return false;
-        }
-      }
-      return write("]");
-    }
     case JSOp::Object: {
-      JSObject* obj = script->getObject(GET_UINT32_INDEX(pc));
+      JSObject* obj = script->getObject(pc);
       RootedValue objv(cx, ObjectValue(*obj));
       JSString* str = ValueToSource(cx, objv);
       if (!str) {
@@ -1948,7 +1998,8 @@ bool ExpressionDecompiler::decompilePC(jsbytecode* pc, uint8_t defIndex) {
     case JSOp::StrictSpreadEval:
       return write("eval(...)");
 
-    case JSOp::New: {
+    case JSOp::New:
+    case JSOp::NewContent: {
       uint16_t argc = GET_ARGC(pc);
       return write("(new ") &&
              decompilePCForStackOperand(pc, -int32_t(argc + 3)) &&
@@ -1999,6 +2050,20 @@ bool ExpressionDecompiler::decompilePC(jsbytecode* pc, uint8_t defIndex) {
       return write("[bigint]");
 #endif
 
+    case JSOp::BuiltinObject: {
+      auto kind = BuiltinObjectKind(GET_UINT8(pc));
+      return write(BuiltinObjectName(kind));
+    }
+
+#ifdef ENABLE_RECORD_TUPLE
+    case JSOp::InitTuple:
+      return write("#[]");
+
+    case JSOp::AddTupleElement:
+    case JSOp::FinishTuple:
+      return write("#[...]");
+#endif
+
     default:
       break;
   }
@@ -2026,24 +2091,21 @@ bool ExpressionDecompiler::decompilePC(jsbytecode* pc, uint8_t defIndex) {
       case JSOp::CallSiteObj:
         return write("OBJ");
 
-      case JSOp::ClassConstructor:
-      case JSOp::DerivedConstructor:
-        return write("CONSTRUCTOR");
-
       case JSOp::Double:
         return sprinter.printf("%lf", GET_INLINE_VALUE(pc).toDouble());
 
       case JSOp::Exception:
         return write("EXCEPTION");
 
-      case JSOp::Finally:
+      case JSOp::Try:
+        // Used for the values live on entry to the finally block.
+        // See TryNoteKind::Finally above.
         if (defIndex == 0) {
-          return write("THROWING");
+          return write("PC");
         }
         MOZ_ASSERT(defIndex == 1);
-        return write("PC");
+        return write("THROWING");
 
-      case JSOp::GImplicitThis:
       case JSOp::FunctionThis:
       case JSOp::ImplicitThis:
         return write("THIS");
@@ -2076,11 +2138,13 @@ bool ExpressionDecompiler::decompilePC(jsbytecode* pc, uint8_t defIndex) {
       case JSOp::IsConstructing:
         return write("JS_IS_CONSTRUCTING");
 
+      case JSOp::IsNullOrUndefined:
+        return write("IS_NULL_OR_UNDEF");
+
       case JSOp::Iter:
         return write("ITER");
 
       case JSOp::Lambda:
-      case JSOp::LambdaArrow:
         return write("FUN");
 
       case JSOp::ToAsyncIter:
@@ -2096,13 +2160,10 @@ bool ExpressionDecompiler::decompilePC(jsbytecode* pc, uint8_t defIndex) {
 
       case JSOp::NewInit:
       case JSOp::NewObject:
-      case JSOp::NewObjectWithGroup:
       case JSOp::ObjWithProto:
         return write("OBJ");
 
       case JSOp::OptimizeSpreadCall:
-        // For stack dump, defIndex == 0 is not used.
-        MOZ_ASSERT(defIndex == 1);
         return write("OPTIMIZED");
 
       case JSOp::Rest:
@@ -2114,8 +2175,8 @@ bool ExpressionDecompiler::decompilePC(jsbytecode* pc, uint8_t defIndex) {
       case JSOp::SuperBase:
         return write("HOMEOBJECTPROTO");
 
-      case JSOp::ToId:
-        return write("TOID(") && decompilePCForStackOperand(pc, -1) &&
+      case JSOp::ToPropertyKey:
+        return write("TOPROPERTYKEY(") && decompilePCForStackOperand(pc, -1) &&
                write(")");
       case JSOp::ToString:
         return write("TOSTRING(") && decompilePCForStackOperand(pc, -1) &&
@@ -2146,6 +2207,15 @@ bool ExpressionDecompiler::decompilePC(jsbytecode* pc, uint8_t defIndex) {
       case JSOp::AsyncResolve:
         return write("PROMISE");
 
+      case JSOp::CheckPrivateField:
+        return write("HasPrivateField");
+
+      case JSOp::NewPrivateName:
+        return write("PRIVATENAME");
+
+      case JSOp::CheckReturn:
+        return write("RVAL");
+
       default:
         break;
     }
@@ -2201,6 +2271,9 @@ bool ExpressionDecompiler::write(JSString* str) {
   if (str == cx->names().dotThis) {
     return write("this");
   }
+  if (str == cx->names().dotNewTarget) {
+    return write("new.target");
+  }
   return sprinter.putString(str);
 }
 
@@ -2212,6 +2285,10 @@ JSAtom* ExpressionDecompiler::loadAtom(jsbytecode* pc) {
   return script->getAtom(pc);
 }
 
+JSString* ExpressionDecompiler::loadString(jsbytecode* pc) {
+  return script->getString(pc);
+}
+
 JSAtom* ExpressionDecompiler::getArg(unsigned slot) {
   MOZ_ASSERT(script->isFunction());
   MOZ_ASSERT(slot < script->numArgs());
@@ -2335,14 +2412,14 @@ static bool DecompileExpressionFromStack(JSContext* cx, int spindex,
 
   *res = nullptr;
 
-#ifdef JS_MORE_DETERMINISTIC
   /*
    * Give up if we need deterministic behavior for differential testing.
    * IonMonkey doesn't use InterpreterFrames and this ensures we get the same
    * error messages.
    */
-  return true;
-#endif
+  if (js::SupportDifferentialTesting()) {
+    return true;
+  }
 
   if (spindex == JSDVG_IGNORE_STACK) {
     return true;
@@ -2351,7 +2428,7 @@ static bool DecompileExpressionFromStack(JSContext* cx, int spindex,
   FrameIter frameIter(cx);
 
   if (frameIter.done() || !frameIter.hasScript() ||
-      frameIter.realm() != cx->realm()) {
+      frameIter.realm() != cx->realm() || frameIter.inPrologue()) {
     return true;
   }
 
@@ -2368,11 +2445,6 @@ static bool DecompileExpressionFromStack(JSContext* cx, int spindex,
 
   MOZ_ASSERT(script->containsPC(valuepc));
 
-  // Give up if in prologue.
-  if (valuepc < script->main()) {
-    return true;
-  }
-
   LifoAllocScope allocScope(&cx->tempLifoAlloc());
   BytecodeParser parser(cx, allocScope.alloc(), frameIter.script());
   if (!parser.parse()) {
@@ -2433,10 +2505,10 @@ static bool DecompileArgumentFromStack(JSContext* cx, int formalIndex,
 
   *res = nullptr;
 
-#ifdef JS_MORE_DETERMINISTIC
   /* See note in DecompileExpressionFromStack. */
-  return true;
-#endif
+  if (js::SupportDifferentialTesting()) {
+    return true;
+  }
 
   /*
    * Settle on the nearest script frame, which should be the builtin that
@@ -2467,7 +2539,8 @@ static bool DecompileArgumentFromStack(JSContext* cx, int formalIndex,
 
   /* Don't handle getters, setters or calls from fun.call/fun.apply. */
   JSOp op = JSOp(*current);
-  if (op != JSOp::Call && op != JSOp::CallIgnoresRv && op != JSOp::New) {
+  if (op != JSOp::Call && op != JSOp::CallContent &&
+      op != JSOp::CallIgnoresRv && op != JSOp::New && op != JSOp::NewContent) {
     return true;
   }
 
@@ -2481,7 +2554,7 @@ static bool DecompileArgumentFromStack(JSContext* cx, int formalIndex,
     return false;
   }
 
-  bool pushedNewTarget = op == JSOp::New;
+  bool pushedNewTarget = op == JSOp::New || op == JSOp::NewContent;
   int formalStackIndex = parser.stackDepthAtPC(current) - GET_ARGC(current) -
                          pushedNewTarget + formalIndex;
   MOZ_ASSERT(formalStackIndex >= 0);
@@ -2509,7 +2582,7 @@ JSString* js::DecompileArgument(JSContext* cx, int formalIndex, HandleValue v) {
     }
     if (result && strcmp(result.get(), "(intermediate value)")) {
       JS::ConstUTF8CharsZ utf8chars(result.get(), strlen(result.get()));
-      return NewStringCopyUTF8Z<CanGC>(cx, utf8chars);
+      return NewStringCopyUTF8Z(cx, utf8chars);
     }
   }
   if (v.isUndefined()) {
@@ -2563,7 +2636,7 @@ static void ReleaseScriptCounts(JSRuntime* rt) {
   rt->scriptAndCountsVector = nullptr;
 }
 
-JS_FRIEND_API void js::StartPCCountProfiling(JSContext* cx) {
+void JS::StartPCCountProfiling(JSContext* cx) {
   JSRuntime* rt = cx->runtime();
 
   if (rt->profilingScripts) {
@@ -2574,12 +2647,12 @@ JS_FRIEND_API void js::StartPCCountProfiling(JSContext* cx) {
     ReleaseScriptCounts(rt);
   }
 
-  ReleaseAllJITCode(rt->defaultFreeOp());
+  ReleaseAllJITCode(rt->gcContext());
 
   rt->profilingScripts = true;
 }
 
-JS_FRIEND_API void js::StopPCCountProfiling(JSContext* cx) {
+void JS::StopPCCountProfiling(JSContext* cx) {
   JSRuntime* rt = cx->runtime();
 
   if (!rt->profilingScripts) {
@@ -2587,7 +2660,7 @@ JS_FRIEND_API void js::StopPCCountProfiling(JSContext* cx) {
   }
   MOZ_ASSERT(!rt->scriptAndCountsVector);
 
-  ReleaseAllJITCode(rt->defaultFreeOp());
+  ReleaseAllJITCode(rt->gcContext());
 
   auto* vec = cx->new_<PersistentRooted<ScriptAndCountsVector>>(
       cx, ScriptAndCountsVector());
@@ -2609,7 +2682,7 @@ JS_FRIEND_API void js::StopPCCountProfiling(JSContext* cx) {
   rt->scriptAndCountsVector = vec;
 }
 
-JS_FRIEND_API void js::PurgePCCounts(JSContext* cx) {
+void JS::PurgePCCounts(JSContext* cx) {
   JSRuntime* rt = cx->runtime();
 
   if (!rt->scriptAndCountsVector) {
@@ -2620,7 +2693,7 @@ JS_FRIEND_API void js::PurgePCCounts(JSContext* cx) {
   ReleaseScriptCounts(rt);
 }
 
-JS_FRIEND_API size_t js::GetPCCountScriptCount(JSContext* cx) {
+size_t JS::GetPCCountScriptCount(JSContext* cx) {
   JSRuntime* rt = cx->runtime();
 
   if (!rt->scriptAndCountsVector) {
@@ -2630,8 +2703,8 @@ JS_FRIEND_API size_t js::GetPCCountScriptCount(JSContext* cx) {
   return rt->scriptAndCountsVector->length();
 }
 
-static MOZ_MUST_USE bool JSONStringProperty(Sprinter& sp, JSONPrinter& json,
-                                            const char* name, JSString* str) {
+[[nodiscard]] static bool JSONStringProperty(Sprinter& sp, JSONPrinter& json,
+                                             const char* name, JSString* str) {
   json.beginStringProperty(name);
   if (!JSONQuoteString(&sp, str)) {
     return false;
@@ -2640,8 +2713,7 @@ static MOZ_MUST_USE bool JSONStringProperty(Sprinter& sp, JSONPrinter& json,
   return true;
 }
 
-JS_FRIEND_API JSString* js::GetPCCountScriptSummary(JSContext* cx,
-                                                    size_t index) {
+JSString* JS::GetPCCountScriptSummary(JSContext* cx, size_t index) {
   JSRuntime* rt = cx->runtime();
 
   if (!rt->scriptAndCountsVector ||
@@ -2663,11 +2735,17 @@ JS_FRIEND_API JSString* js::GetPCCountScriptSummary(JSContext* cx,
 
   json.beginObject();
 
-  RootedString filename(cx, NewStringCopyZ<CanGC>(cx, script->filename()));
-  if (!filename) {
+  Rooted<JSString*> filenameStr(cx);
+  if (const char* filename = script->filename()) {
+    filenameStr =
+        JS_NewStringCopyUTF8N(cx, JS::UTF8Chars(filename, strlen(filename)));
+  } else {
+    filenameStr = JS_GetEmptyString(cx);
+  }
+  if (!filenameStr) {
     return nullptr;
   }
-  if (!JSONStringProperty(sp, json, "file", filename)) {
+  if (!JSONStringProperty(sp, json, "file", filenameStr)) {
     return nullptr;
   }
   json.property("line", script->lineno());
@@ -2682,9 +2760,9 @@ JS_FRIEND_API JSString* js::GetPCCountScriptSummary(JSContext* cx,
 
   uint64_t total = 0;
 
-  jsbytecode* codeEnd = script->codeEnd();
-  for (jsbytecode* pc = script->code(); pc < codeEnd; pc = GetNextPc(pc)) {
-    if (const PCCounts* counts = sac.maybeGetPCCounts(pc)) {
+  AllBytecodesIterable iter(script);
+  for (BytecodeLocation loc : iter) {
+    if (const PCCounts* counts = sac.maybeGetPCCounts(loc.toRawBytecode())) {
       total += counts->numExec();
     }
   }
@@ -2777,7 +2855,7 @@ static bool GetPCCountJSON(JSContext* cx, const ScriptAndCounts& sac,
       }
 
       JS::ConstUTF8CharsZ utf8chars(text.get(), strlen(text.get()));
-      JSString* str = NewStringCopyUTF8Z<CanGC>(cx, utf8chars);
+      JSString* str = NewStringCopyUTF8Z(cx, utf8chars);
       if (!str) {
         return false;
       }
@@ -2845,11 +2923,10 @@ static bool GetPCCountJSON(JSContext* cx, const ScriptAndCounts& sac,
 
   json.endObject();
 
-  return true;
+  return !sp.hadOutOfMemory();
 }
 
-JS_FRIEND_API JSString* js::GetPCCountScriptContents(JSContext* cx,
-                                                     size_t index) {
+JSString* JS::GetPCCountScriptContents(JSContext* cx, size_t index) {
   JSRuntime* rt = cx->runtime();
 
   if (!rt->scriptAndCountsVector ||
@@ -2921,6 +2998,7 @@ static bool GenerateLcovInfo(JSContext* cx, JS::Realm* realm,
     CollectedScripts result(&queue);
     IterateScripts(cx, realm, &result, &CollectedScripts::consider);
     if (!result.ok) {
+      ReportOutOfMemory(cx);
       return false;
     }
   }
@@ -2946,6 +3024,7 @@ static bool GenerateLcovInfo(JSContext* cx, JS::Realm* realm,
     }
 
     if (!coverage::CollectScriptCoverage(script, false)) {
+      ReportOutOfMemory(cx);
       return false;
     }
 
@@ -2998,7 +3077,7 @@ static bool GenerateLcovInfo(JSContext* cx, JS::Realm* realm,
   return true;
 }
 
-JS_FRIEND_API UniqueChars js::GetCodeCoverageSummaryAll(JSContext* cx,
+JS_PUBLIC_API UniqueChars js::GetCodeCoverageSummaryAll(JSContext* cx,
                                                         size_t* length) {
   Sprinter out(cx);
   if (!out.init()) {
@@ -3007,7 +3086,6 @@ JS_FRIEND_API UniqueChars js::GetCodeCoverageSummaryAll(JSContext* cx,
 
   for (RealmsIter realm(cx->runtime()); !realm.done(); realm.next()) {
     if (!GenerateLcovInfo(cx, realm, out)) {
-      JS_ReportOutOfMemory(cx);
       return nullptr;
     }
   }
@@ -3016,7 +3094,7 @@ JS_FRIEND_API UniqueChars js::GetCodeCoverageSummaryAll(JSContext* cx,
   return js::DuplicateString(cx, out.string(), *length);
 }
 
-JS_FRIEND_API UniqueChars js::GetCodeCoverageSummary(JSContext* cx,
+JS_PUBLIC_API UniqueChars js::GetCodeCoverageSummary(JSContext* cx,
                                                      size_t* length) {
   Sprinter out(cx);
   if (!out.init()) {
@@ -3024,67 +3102,9 @@ JS_FRIEND_API UniqueChars js::GetCodeCoverageSummary(JSContext* cx,
   }
 
   if (!GenerateLcovInfo(cx, cx->realm(), out)) {
-    JS_ReportOutOfMemory(cx);
     return nullptr;
   }
 
   *length = out.getOffset();
   return js::DuplicateString(cx, out.string(), *length);
 }
-
-bool js::GetSuccessorBytecodes(JSScript* script, jsbytecode* pc,
-                               PcVector& successors) {
-  MOZ_ASSERT(script->containsPC(pc));
-
-  JSOp op = (JSOp)*pc;
-  if (BytecodeFallsThrough(op)) {
-    if (!successors.append(GetNextPc(pc))) {
-      return false;
-    }
-  }
-
-  if (CodeSpec(op).type() == JOF_JUMP) {
-    if (!successors.append(pc + GET_JUMP_OFFSET(pc))) {
-      return false;
-    }
-  } else if (op == JSOp::TableSwitch) {
-    if (!successors.append(pc + GET_JUMP_OFFSET(pc))) {
-      return false;
-    }
-    jsbytecode* npc = pc + JUMP_OFFSET_LEN;
-
-    int32_t low = GET_JUMP_OFFSET(npc);
-    npc += JUMP_OFFSET_LEN;
-    int ncases = GET_JUMP_OFFSET(npc) - low + 1;
-    npc += JUMP_OFFSET_LEN;
-
-    for (int i = 0; i < ncases; i++) {
-      if (!successors.append(script->tableSwitchCasePC(pc, i))) {
-        return false;
-      }
-    }
-  }
-
-  return true;
-}
-
-bool js::GetPredecessorBytecodes(JSScript* script, jsbytecode* pc,
-                                 PcVector& predecessors) {
-  MOZ_ASSERT(js::BytecodeLocation(script, pc).isInBounds(script));
-
-  for (const BytecodeLocation& loc : js::AllBytecodesIterable(script)) {
-    PcVector successors;
-    if (!GetSuccessorBytecodes(script, loc.toRawBytecode(), successors)) {
-      return false;
-    }
-    for (size_t i = 0; i < successors.length(); i++) {
-      if (successors[i] == pc) {
-        if (!predecessors.append(loc.toRawBytecode())) {
-          return false;
-        }
-        break;
-      }
-    }
-  }
-  return true;
-}
