@@ -2,20 +2,18 @@
 # License, v. 2.0. If a copy of the MPL was not distributed with this
 # file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
-from __future__ import absolute_import, unicode_literals
-
 import logging
 import time
 import unittest
 
-from mach.logging import StructuredHumanFormatter
-
 from mozunit import main
 
+from mach.logging import StructuredHumanFormatter
+
 
 class DummyLogger(logging.Logger):
     def __init__(self, cb):
-        logging.Logger.__init__(self, 'test')
+        logging.Logger.__init__(self, "test")
 
         self._cb = cb
 
@@ -33,15 +31,18 @@ class TestStructuredHumanFormatter(unittest.TestCase):
             result = formatter.format(record)
             relevant = result[9:]
 
-            self.assertEqual(relevant, 'Test: s\xe9curit\xe9')
+            self.assertEqual(relevant, "Test: s\xe9curit\xe9")
 
         logger = DummyLogger(on_record)
 
-        value = 's\xe9curit\xe9'
+        value = "s\xe9curit\xe9"
 
-        logger.log(logging.INFO, 'Test: {utf}',
-            extra={'action': 'action', 'params': {'utf': value}})
+        logger.log(
+            logging.INFO,
+            "Test: {utf}",
+            extra={"action": "action", "params": {"utf": value}},
+        )
 
 
-if __name__ == '__main__':
+if __name__ == "__main__":
     main()
