@@ -1,4 +1,4 @@
-/* Copyright (C) 2011 Wildfire Games.
+/* Copyright (C) 2022 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -23,14 +23,13 @@
 #include "graphics/ParticleEmitter.h"
 #include "graphics/ParticleManager.h"
 #include "graphics/TextureManager.h"
-#include "lib/rand.h"
 #include "maths/MathUtil.h"
 #include "ps/CLogger.h"
 #include "ps/Filesystem.h"
 #include "ps/XML/Xeromyces.h"
 #include "renderer/Renderer.h"
 
-#include <boost/random/uniform_real.hpp>
+#include <boost/random/uniform_real_distribution.hpp>
 
 
 /**
@@ -120,7 +119,7 @@ public:
 
 	virtual float Compute(CParticleEmitterType& type, CParticleEmitter& UNUSED(emitter))
 	{
-		return boost::uniform_real<>(m_Min, m_Max)(type.m_Manager.m_RNG);
+		return boost::random::uniform_real_distribution<float>(m_Min, m_Max)(type.m_Manager.m_RNG);
 	}
 
 	virtual float Min(CParticleEmitterType& UNUSED(type))
@@ -329,7 +328,7 @@ int CParticleEmitterType::GetVariableID(const std::string& name)
 	if (name == "color.r")			return VAR_COLOR_R;
 	if (name == "color.g")			return VAR_COLOR_G;
 	if (name == "color.b")			return VAR_COLOR_B;
-	LOGWARNING(L"Particle sets unknown variable '%hs'", name.c_str());
+	LOGWARNING("Particle sets unknown variable '%s'", name.c_str());
 	return -1;
 }
 
@@ -353,20 +352,16 @@ bool CParticleEmitterType::LoadXML(const VfsPath& path)
 	m_Variables[VAR_COLOR_R] = IParticleVarPtr(new CParticleVarConstant(1.f));
 	m_Variables[VAR_COLOR_G] = IParticleVarPtr(new CParticleVarConstant(1.f));
 	m_Variables[VAR_COLOR_B] = IParticleVarPtr(new CParticleVarConstant(1.f));
-	m_BlendEquation = GL_FUNC_ADD;
-	m_BlendFuncSrc = GL_SRC_ALPHA;
-	m_BlendFuncDst = GL_ONE_MINUS_SRC_ALPHA;
+	m_BlendMode = BlendMode::ADD;
 	m_StartFull = false;
 	m_UseRelativeVelocity = false;
 	m_Texture = g_Renderer.GetTextureManager().GetErrorTexture();
 
 	CXeromyces XeroFile;
-	PSRETURN ret = XeroFile.Load(g_VFS, path);
+	PSRETURN ret = XeroFile.Load(g_VFS, path, "particle");
 	if (ret != PSRETURN_OK)
 		return false;
 
-	// TODO: should do some RNG schema validation
-
 	// Define all the elements and attributes used in the XML file
 #define EL(x) int el_##x = XeroFile.GetElementID(#x)
 #define AT(x) int at_##x = XeroFile.GetAttributeID(#x)
@@ -399,36 +394,21 @@ bool CParticleEmitterType::LoadXML(const VfsPath& path)
 		if (Child.GetNodeName() == el_texture)
 		{
 			CTextureProperties textureProps(Child.GetText().FromUTF8());
-			textureProps.SetWrap(GL_CLAMP_TO_EDGE);
+			textureProps.SetAddressMode(
+				Renderer::Backend::Sampler::AddressMode::CLAMP_TO_EDGE);
 			m_Texture = g_Renderer.GetTextureManager().CreateTexture(textureProps);
 		}
 		else if (Child.GetNodeName() == el_blend)
 		{
-			CStr mode = Child.GetAttributes().GetNamedItem(at_mode);
+			const CStr mode = Child.GetAttributes().GetNamedItem(at_mode);
 			if (mode == "add")
-			{
-				m_BlendEquation = GL_FUNC_ADD;
-				m_BlendFuncSrc = GL_SRC_ALPHA;
-				m_BlendFuncDst = GL_ONE;
-			}
+				m_BlendMode = BlendMode::ADD;
 			else if (mode == "subtract")
-			{
-				m_BlendEquation = GL_FUNC_REVERSE_SUBTRACT;
-				m_BlendFuncSrc = GL_SRC_ALPHA;
-				m_BlendFuncDst = GL_ONE;
-			}
+				m_BlendMode = BlendMode::SUBTRACT;
 			else if (mode == "over")
-			{
-				m_BlendEquation = GL_FUNC_ADD;
-				m_BlendFuncSrc = GL_SRC_ALPHA;
-				m_BlendFuncDst = GL_ONE_MINUS_SRC_ALPHA;
-			}
+				m_BlendMode = BlendMode::OVERLAY;
 			else if (mode == "multiply")
-			{
-				m_BlendEquation = GL_FUNC_ADD;
-				m_BlendFuncSrc = GL_ZERO;
-				m_BlendFuncDst = GL_ONE_MINUS_SRC_COLOR;
-			}
+				m_BlendMode = BlendMode::MULTIPLY;
 		}
 		else if (Child.GetNodeName() == el_start_full)
 		{
@@ -589,8 +569,8 @@ void CParticleEmitterType::UpdateEmitterStep(CParticleEmitter& emitter, float dt
 		// TODO: this should probably be done as a variable or something,
 		// instead of hardcoding
 		float ageFrac = p.age / p.maxAge;
-		float a = std::min(1.f-ageFrac, 5.f*ageFrac);
-		p.color.A = clamp((int)(a*255.f), 0, 255);
+		float a = std::min(1.f - ageFrac, 5.f * ageFrac);
+		p.color.A = Clamp(static_cast<int>(a * 255.f), 0, 255);
 	}
 
 	for (size_t i = 0; i < m_Effectors.size(); ++i)
