@@ -1,4 +1,4 @@
-/* Copyright (C) 2013 Wildfire Games.
+/* Copyright (C) 2024 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -16,110 +16,64 @@
  */
 
 /*
-CGUI
-
---Overview--
-
-	This is the top class of the whole GUI, all objects
-	and settings are stored within this class.
-
---More info--
-
-	Check GUI.h
-
-*/
+ * This is the top class of the whole GUI, all objects
+ * and settings are stored within this class.
+ */
 
 #ifndef INCLUDED_CGUI
 #define INCLUDED_CGUI
 
-//--------------------------------------------------------
-//  Includes / Compiler directives
-//--------------------------------------------------------
-// NOTE: GUI.h included at the bottom of this file (has to be after CGUI class
-// definition)
-
-#include "GUITooltip.h"
-#include "GUIbase.h"
-
-#include "ps/Overlay.h" // CPos and CRect
-
+#include "gui/GUITooltip.h"
+#include "gui/SettingTypes/CGUIColor.h"
+#include "gui/SGUIIcon.h"
+#include "gui/SGUIMessage.h"
+#include "gui/SGUIStyle.h"
 #include "lib/input.h"
-
+#include "maths/Rect.h"
+#include "maths/Size2D.h"
+#include "maths/Vector2D.h"
 #include "ps/XML/Xeromyces.h"
+#include "scriptinterface/ScriptForward.h"
 
-#include <boost/unordered_set.hpp>
+#include <map>
+#include <memory>
+#include <unordered_map>
+#include <unordered_set>
+#include <vector>
 
-//--------------------------------------------------------
-//  Macros
-//--------------------------------------------------------
+extern const double SELECT_DBLCLICK_RATE;
 
-//--------------------------------------------------------
-//  Types
-//--------------------------------------------------------
-
-//--------------------------------------------------------
-//  Error declarations
-//--------------------------------------------------------
-
-ERROR_TYPE(GUI, JSOpenFailed);
-
-//--------------------------------------------------------
-//  Declarations
-//--------------------------------------------------------
-
-/**
- * Contains a list of values for new defaults to objects.
- */
-struct SGUIStyle
-{
-	// A list of defaults for 
-	std::map<CStr, CStrW> m_SettingsDefaults;
-};
-
-struct JSObject; // The GUI stores a JSObject*, so needs to know that JSObject exists
-class IGUIObject;
+class CCanvas2D;
 class CGUISpriteInstance;
-struct SGUIText;
-struct CColor;
-struct SGUIText;
-struct SGUIIcon;
-class CGUIString;
 class CGUISprite;
+class IGUIObject;
 struct SGUIImageEffects;
 struct SGUIScrollBarStyle;
-class GUITooltip;
+
+class GUIProxyProps;
+
+using map_pObjects = std::map<CStr, IGUIObject*>;
 
 /**
  * The main object that represents a whole GUI page.
- *
- * No interfacial functions throws.
  */
 class CGUI
 {
 	NONCOPYABLE(CGUI);
-	
-	friend class IGUIObject;
-	friend class IGUIScrollBarOwner;
-	friend class CInternalCGUIAccessorBase;
 
 private:
 	// Private typedefs
-	typedef IGUIObject *(*ConstructObjectFunction)();
+	using ConstructObjectFunction = IGUIObject* (*)(CGUI&);
 
 public:
-	CGUI();
+	CGUI(ScriptContext& context);
 	~CGUI();
 
 	/**
-	 * Initializes GUI script classes
+	 * Informs the GUI page which GUI object types may be constructed from XML.
 	 */
-	static void ScriptingInit();
+	void AddObjectTypes();
 
-	/**
-	 * Initializes the GUI, needs to be called before the GUI is used
-	 */
-	void Initialize();
-	
 	/**
 	 * Performs processing that should happen every frame
 	 * (including sending the "Tick" event to scripts)
@@ -129,46 +83,33 @@ public:
 	/**
 	 * Sends a specified script event to every object
 	 *
-	 * @param EventName String representation of event name
+	 * @param eventName String representation of event name
 	 */
-	void SendEventToAll(const CStr& EventName);
+	void SendEventToAll(const CStr& eventName);
+
+	/**
+	 * Sends a specified script event to every object
+	 *
+	 * @param eventName String representation of event name
+	 * @param paramData JS::HandleValueArray storing the arguments passed to the event handler.
+	 */
+	void SendEventToAll(const CStr& eventName, const JS::HandleValueArray& paramData);
 
 	/**
 	 * Displays the whole GUI
 	 */
-	void Draw();
+	void Draw(CCanvas2D& canvas);
 
 	/**
 	 * Draw GUI Sprite
 	 *
 	 * @param Sprite Object referring to the sprite (which also caches
 	 *        calculations for faster rendering)
-	 * @param CellID Number of the icon cell to use. (Ignored if this sprite doesn't
-	 *        have any images with "cell-size")
-	 * @param Z Drawing order, depth value
+	 * @param Canvas Canvas to draw on
 	 * @param Rect Position and Size
 	 * @param Clipping The sprite shouldn't be drawn outside this rectangle
 	 */
-	void DrawSprite(const CGUISpriteInstance& Sprite, int CellID, const float &Z, 
-					const CRect &Rect, const CRect &Clipping=CRect());
-
-	/**
-	 * Draw a SGUIText object
-	 *
-	 * @param Text Text object.
-	 * @param DefaultColor Color used if no tag applied.
-	 * @param pos position
-	 * @param z z value.
-	 * @param clipping
-	 */
-	void DrawText(SGUIText &Text, const CColor &DefaultColor, 
-				  const CPos &pos, const float &z, const CRect &clipping);
-
-	/**
-	 * Clean up, call this to clean up all memory allocated
-	 * within the GUI.
-	 */
-	void Destroy();
+	void DrawSprite(const CGUISpriteInstance& Sprite, CCanvas2D& canvas, const CRect& Rect, const CRect& Clipping = CRect());
 
 	/**
 	 * The replacement of Process(), handles an SDL_Event_
@@ -186,7 +127,35 @@ public:
 	 * @param Filename Name of file
 	 * @param Paths Set of paths; all XML and JS files loaded will be added to this
 	 */
-	void LoadXmlFile(const VfsPath& Filename, boost::unordered_set<VfsPath>& Paths);
+	void LoadXmlFile(const VfsPath& Filename, std::unordered_set<VfsPath>& Paths);
+
+	/**
+	 * Called after all XML files linked in the page file were loaded.
+	 */
+	void LoadedXmlFiles();
+
+	/**
+	 * Allows the JS side to modify the hotkey setting assigned to a GUI object.
+	 */
+	void SetObjectHotkey(IGUIObject* pObject, const CStr& hotkeyTag);
+	void UnsetObjectHotkey(IGUIObject* pObject, const CStr& hotkeyTag);
+
+	/**
+	 * Allows the JS side to modify the style setting assigned to a GUI object.
+	 */
+	void SetObjectStyle(IGUIObject* pObject, const CStr& styleName);
+	void UnsetObjectStyle(IGUIObject* pObject);
+
+	/**
+	 * Allows the JS side to add or remove global hotkeys.
+	 */
+	void SetGlobalHotkey(const CStr& hotkeyTag, const CStr& eventName, JS::HandleValue function);
+	void UnsetGlobalHotkey(const CStr& hotkeyTag, const CStr& eventName);
+
+	/**
+	 * Return the object which is an ancestor of every other GUI object.
+	 */
+	IGUIObject* GetBaseObject();
 
 	/**
 	 * Checks if object exists and return true or false accordingly
@@ -196,26 +165,42 @@ public:
 	 */
 	bool ObjectExists(const CStr& Name) const;
 
-
 	/**
-	 * Returns the GUI object with the desired name, or NULL
+	 * Returns the GUI object with the desired name, or nullptr
 	 * if no match is found,
 	 *
 	 * @param Name String name of object
-	 * @return Matching object, or NULL
+	 * @return Matching object, or nullptr
 	 */
 	IGUIObject* FindObjectByName(const CStr& Name) const;
 
 	/**
-	 * Returns the GUI object under the mouse, or NULL if none.
+	 * Returns the GUI object under the mouse, or nullptr if none.
 	 */
-	IGUIObject* FindObjectUnderMouse() const;
+	IGUIObject* FindObjectUnderMouse();
+
+	/**
+	 * Returns the current screen coordinates of the cursor.
+	 */
+	const CVector2D& GetMousePos() const { return m_MousePos; }
+
+	/**
+	 * Returns the currently pressed mouse buttons.
+	 */
+	const unsigned int& GetMouseButtons() { return m_MouseButtons; }
+
+	const SGUIScrollBarStyle* GetScrollBarStyle(const CStr& style) const;
+
+	/**
+	 * Returns the current GUI window size.
+	 */
+	CSize2D GetWindowSize() const;
 
 	/**
 	 * The GUI needs to have all object types inputted and
 	 * their constructors. Also it needs to associate a type
 	 * by a string name of the type.
-	 * 
+	 *
 	 * To add a type:
 	 * @code
 	 * AddObjectType("button", &CButton::ConstructObject);
@@ -239,72 +224,46 @@ public:
 	 */
 	void UpdateResolution();
 
-	/**
-	 * Generate a SGUIText object from the inputted string.
-	 * The function will break down the string and its
-	 * tags to calculate exactly which rendering queries
-	 * will be sent to the Renderer. Also, horizontal alignment
-	 * is taken into acount in this method but NOT vertical alignment.
-	 *
-	 * Done through the CGUI since it can communicate with 
-	 *
-	 * @param Text Text to generate SGUIText object from
-	 * @param Font Default font, notice both Default color and default font
-	 *		  can be changed by tags.
-	 * @param Width Width, 0 if no word-wrapping.
-	 * @param BufferZone space between text and edge, and space between text and images.
-	 * @param pObject Optional parameter for error output. Used *only* if error parsing fails, 
-	 *		  and we need to be able to output which object the error occured in to aid the user. 
-	 */
-	SGUIText GenerateText(const CGUIString &Text, const CStrW& Font,
-						  const float &Width, const float &BufferZone,
-						  const IGUIObject *pObject=NULL);
-
-	/**
-	 * Returns the JSObject* associated with the GUI
-	 *
-	 * @return The relevant JS object
-	 */
-	JSObject* GetScriptObject() { return m_ScriptObject; }
-
 	/**
 	 * Check if an icon exists
 	 */
-	bool IconExists(const CStr& str) const { return (m_Icons.count(str) != 0); }
+	bool HasIcon(const CStr& name) const { return (m_Icons.find(name) != m_Icons.end()); }
 
 	/**
-	 * Get Icon (a copy, can never be changed)
+	 * Get Icon (a const reference, can never be changed)
 	 */
-	SGUIIcon GetIcon(const CStr& str) const { return m_Icons.find(str)->second; }
+	const SGUIIcon& GetIcon(const CStr& name) const { return m_Icons.at(name); }
 
 	/**
-	 * Get pre-defined color (if it exists)
-	 * Returns false if it fails.
+	 * Check if a style exists
 	 */
-	bool GetPreDefinedColor(const CStr& name, CColor &Output);
+	bool HasStyle(const CStr& name) const { return (m_Styles.find(name) != m_Styles.end()); }
+
+	/**
+	 * Get Style if it exists, otherwise throws an exception.
+	 */
+	const SGUIStyle& GetStyle(const CStr& name) const { return m_Styles.at(name); }
+
+	/**
+	 * Check if a predefined color of that name exists.
+	 */
+	bool HasPreDefinedColor(const CStr& name) const { return (m_PreDefinedColors.find(name) != m_PreDefinedColors.end()); }
+
+	/**
+	 * Resolve the predefined color if it exists, otherwise throws an exception.
+	 */
+	const CGUIColor& GetPreDefinedColor(const CStr& name) const { return m_PreDefinedColors.at(name); }
+
+	GUIProxyProps* GetProxyData(const js::BaseProxyHandler* ptr) { return m_ProxyData.at(ptr).get(); }
+
+	std::shared_ptr<ScriptInterface> GetScriptInterface() { return m_ScriptInterface; };
 
 private:
-
 	/**
-	 * Updates the object pointers, needs to be called each
-	 * time an object has been added or removed.
-	 *
-	 * This function is atomic, meaning if it throws anything, it will
-	 * have seen it through that nothing was ultimately changed.
-	 *
-	 * @throws PSERROR_GUI that is thrown from IGUIObject::AddToPointersMap().
+	 * The CGUI takes ownership of the child object and links the parent with the child.
+	 * Returns false on failure to take over ownership of the child object.
 	 */
-	void UpdateObjects();
-
-	/**
-	 * Adds an object to the GUI's object database
-	 * Private, since you can only add objects through 
-	 * XML files. Why? Because it enables the GUI to
-	 * be much more encapsulated and safe.
-	 *
-	 * @throws	Rethrows PSERROR_GUI from IGUIObject::AddChild().
-	 */
-	void AddObject(IGUIObject* pObject);
+	bool AddObject(IGUIObject& parent, IGUIObject& child);
 
 	/**
 	 * You input the name of the object type, and let's
@@ -314,22 +273,41 @@ private:
 	 * @param str Name of object type
 	 * @return Newly constructed IGUIObject (but constructed as a subclass)
 	 */
-	IGUIObject *ConstructObject(const CStr& str);
-
-	/**
-	 * Get Focused Object.
-	 */
-	IGUIObject *GetFocusedObject() { return m_FocusedObject; }
+	IGUIObject* ConstructObject(const CStr& str);
 
 public:
+	/**
+	 * Get Focused Object.
+	 */
+	IGUIObject* GetFocusedObject() { return m_FocusedObject; }
+
 	/**
 	 * Change focus to new object.
 	 * Will send LOST_FOCUS/GOT_FOCUS messages as appropriate.
-	 * pObject can be NULL to remove all focus.
+	 * pObject can be nullptr to remove all focus.
 	 */
 	void SetFocusedObject(IGUIObject* pObject);
 
+	/**
+	 * Alert the focussed object of this GUIPage that the focus of the page has changed.
+	 */
+	void SendFocusMessage(EGUIMessageType msg);
+
+	/**
+	 * Reads a string value and modifies the given value of type T if successful.
+	 * Does not change the value upon conversion failure.
+	 *
+	 * @param pGUI The GUI page which may contain data relevant to the parsing
+	 *             (for example predefined colors).
+	 * @param Value The value in string form, like "0 0 100% 100%"
+	 * @param tOutput Parsed value of type T
+	 * @return True at success.
+	 */
+	template <typename T>
+	static bool ParseString(const CGUI* pGUI, const CStrW& Value, T& tOutput);
+
 private:
+
 	//--------------------------------------------------------
 	/** @name XML Reading Xeromyces specific subroutines
 	 *
@@ -392,7 +370,7 @@ private:
 	 *
 	 * @see LoadXmlFile()
 	 */
-	void Xeromyces_ReadRootObjects(XMBElement Element, CXeromyces* pFile, boost::unordered_set<VfsPath>& Paths);
+	void Xeromyces_ReadRootObjects(const XMBData& xmb, XMBElement element, std::unordered_set<VfsPath>& Paths);
 
 	/**
 	 * Reads in the root element \<sprites\> (the DOMElement).
@@ -403,7 +381,7 @@ private:
 	 *
 	 * @see LoadXmlFile()
 	 */
-	void Xeromyces_ReadRootSprites(XMBElement Element, CXeromyces* pFile);
+	void Xeromyces_ReadRootSprites(const XMBData& xmb, XMBElement element);
 
 	/**
 	 * Reads in the root element \<styles\> (the DOMElement).
@@ -414,7 +392,7 @@ private:
 	 *
 	 * @see LoadXmlFile()
 	 */
-	void Xeromyces_ReadRootStyles(XMBElement Element, CXeromyces* pFile);
+	void Xeromyces_ReadRootStyles(const XMBData& xmb, XMBElement element);
 
 	/**
 	 * Reads in the root element \<setup\> (the DOMElement).
@@ -425,7 +403,7 @@ private:
 	 *
 	 * @see LoadXmlFile()
 	 */
-	void Xeromyces_ReadRootSetup(XMBElement Element, CXeromyces* pFile);
+	void Xeromyces_ReadRootSetup(const XMBData& xmb, XMBElement element);
 
 	// Read Subs
 
@@ -451,14 +429,15 @@ private:
 	 *
 	 * @see LoadXmlFile()
 	 */
-	void Xeromyces_ReadObject(XMBElement Element, CXeromyces* pFile, IGUIObject *pParent, const std::vector<std::pair<CStr, CStr> >& NameSubst, boost::unordered_set<VfsPath>& Paths);
+	IGUIObject* Xeromyces_ReadObject(const XMBData& xmb, XMBElement element, IGUIObject* pParent, std::vector<std::pair<CStr, CStr> >& NameSubst, std::unordered_set<VfsPath>& Paths, u32 nesting_depth);
 
 	/**
 	 * Reads in the element \<repeat\>, which repeats its child \<object\>s
-	 * 'count' times, replacing the string "[n]" in its descendants' names
-	 * with "[0]", "[1]", etc.
+	 * 'count' times, replacing the string "[n]" (or the value of the attribute
+	 * 'var' enclosed in square brackets) in its descendants' names with "[0]",
+	 * "[1]", etc.
 	 */
-	void Xeromyces_ReadRepeat(XMBElement Element, CXeromyces* pFile, IGUIObject *pParent, boost::unordered_set<VfsPath>& Paths);
+	void Xeromyces_ReadRepeat(const XMBData& xmb, XMBElement element, IGUIObject* pParent, std::vector<std::pair<CStr, CStr> >& NameSubst, std::unordered_set<VfsPath>& Paths, u32 nesting_depth);
 
 	/**
 	 * Reads in the element \<script\> (the XMBElement) and executes
@@ -471,7 +450,7 @@ private:
 	 *
 	 * @see LoadXmlFile()
 	 */
-	void Xeromyces_ReadScript(XMBElement Element, CXeromyces* pFile, boost::unordered_set<VfsPath>& Paths);
+	void Xeromyces_ReadScript(const XMBData& xmb, XMBElement element, std::unordered_set<VfsPath>& Paths);
 
 	/**
 	 * Reads in the element \<sprite\> (the XMBElement) and stores the
@@ -483,7 +462,7 @@ private:
 	 *
 	 * @see LoadXmlFile()
 	 */
-	void Xeromyces_ReadSprite(XMBElement Element, CXeromyces* pFile);
+	void Xeromyces_ReadSprite(const XMBData& xmb, XMBElement element);
 
 	/**
 	 * Reads in the element \<image\> (the XMBElement) and stores the
@@ -496,7 +475,7 @@ private:
 	 *
 	 * @see LoadXmlFile()
 	 */
-	void Xeromyces_ReadImage(XMBElement Element, CXeromyces* pFile, CGUISprite &parent);
+	void Xeromyces_ReadImage(const XMBData& xmb, XMBElement element, CGUISprite& parent);
 
 	/**
 	 * Reads in the element \<effect\> (the XMBElement) and stores the
@@ -509,7 +488,7 @@ private:
 	 *
 	 * @see LoadXmlFile()
 	 */
-	void Xeromyces_ReadEffects(XMBElement Element, CXeromyces* pFile, SGUIImageEffects &effects);
+	void Xeromyces_ReadEffects(const XMBData& xmb, XMBElement element, SGUIImageEffects& effects);
 
 	/**
 	 * Reads in the element \<style\> (the XMBElement) and stores the
@@ -521,7 +500,7 @@ private:
 	 *
 	 * @see LoadXmlFile()
 	 */
-	void Xeromyces_ReadStyle(XMBElement Element, CXeromyces* pFile);
+	void Xeromyces_ReadStyle(const XMBData& xmb, XMBElement element);
 
 	/**
 	 * Reads in the element \<scrollbar\> (the XMBElement) and stores the
@@ -533,7 +512,7 @@ private:
 	 *
 	 * @see LoadXmlFile()
 	 */
-	void Xeromyces_ReadScrollBarStyle(XMBElement Element, CXeromyces* pFile);
+	void Xeromyces_ReadScrollBarStyle(const XMBData& xmb, XMBElement element);
 
 	/**
 	 * Reads in the element \<icon\> (the XMBElement) and stores the
@@ -545,8 +524,8 @@ private:
 	 *
 	 * @see LoadXmlFile()
 	 */
-	void Xeromyces_ReadIcon(XMBElement Element, CXeromyces* pFile);
-	
+	void Xeromyces_ReadIcon(const XMBData& xmb, XMBElement element);
+
 	/**
 	 * Reads in the element \<tooltip\> (the XMBElement) and stores the
 	 * result as an object with the name __tooltip_#.
@@ -557,7 +536,7 @@ private:
 	 *
 	 * @see LoadXmlFile()
 	 */
-	void Xeromyces_ReadTooltip(XMBElement Element, CXeromyces* pFile);
+	void Xeromyces_ReadTooltip(const XMBData& xmb, XMBElement element);
 
 	/**
 	 * Reads in the element \<color\> (the XMBElement) and stores the
@@ -569,7 +548,7 @@ private:
 	 *
 	 * @see LoadXmlFile()
 	 */
-	void Xeromyces_ReadColor(XMBElement Element, CXeromyces* pFile);
+	void Xeromyces_ReadColor(const XMBData& xmb, XMBElement element);
 
 	//@}
 
@@ -582,19 +561,14 @@ private:
 	//--------------------------------------------------------
 	//@{
 
-	/**
-	 * An JSObject* under which all GUI JavaScript things will
-	 * be created, so that they can be garbage-collected
-	 * when the GUI shuts down.
-	 */
-	JSObject* m_ScriptObject;
+	std::shared_ptr<ScriptInterface> m_ScriptInterface;
 
 	/**
-	 * don't want to pass this around with the 
+	 * don't want to pass this around with the
 	 * ChooseMouseOverAndClosest broadcast -
 	 * we'd need to pack this and pNearest in a struct
 	 */
-	CPos m_MousePos;
+	CVector2D m_MousePos;
 
 	/**
 	 * Indicates which buttons are pressed (bit 0 = LMB,
@@ -605,14 +579,6 @@ private:
 	// Tooltip
 	GUITooltip m_Tooltip;
 
-	/**
-	 * This is a bank of custom colors, it is simply a look up table that
-	 * will return a color object when someone inputs the name of that
-	 * color. Of course the colors have to be declared in XML, there are
-	 * no hard-coded values.
-	 */
-	std::map<CStr, CColor>	m_PreDefinedColors;
-
 	//@}
 	//--------------------------------------------------------
 	/** @name Objects */
@@ -623,7 +589,7 @@ private:
 	 * Base Object, all its children are considered parentless
 	 * because this is not a real object per se.
 	 */
-	IGUIObject* m_BaseObject;
+	std::unique_ptr<IGUIObject> m_BaseObject;
 
 	/**
 	 * Focused object!
@@ -632,7 +598,7 @@ private:
 	 */
 	IGUIObject* m_FocusedObject;
 
-	/** 
+	/**
 	 * Just pointers for fast name access, each object
 	 * is really constructed within its parent for easy
 	 * recursive management.
@@ -654,32 +620,81 @@ private:
 	 * IGUIObjects by name... For instance m_ObjectTypes["button"]
 	 * is filled with a function that will "return new CButton();"
 	 */
-	std::map<CStr, ConstructObjectFunction>	m_ObjectTypes;
+	std::map<CStr, ConstructObjectFunction> m_ObjectTypes;
+
+	/**
+	 * This is intended to store the JSFunction when accessing certain properties.
+	 * The problem is that these functions are per-scriptInterface, and proxy handlers aren't.
+	 * So we know what we want to store, but we don't really have anywhere to store it.
+	 * It would be simpler to recreate the functions on every JS call, but that is slower
+	 * (this may or may not matter now and in the future).
+	 * It's not a great solution, but I can't find a better one at the moment.
+	 * An alternative would be to store these on the proxy's prototype,
+	 * but that embarks a lot of un-necessary code.
+	 */
+	std::unordered_map<const js::BaseProxyHandler*, std::unique_ptr<GUIProxyProps>> m_ProxyData;
 
 	/**
 	 * Map from hotkey names to objects that listen to the hotkey.
 	 * (This is an optimisation to avoid recursing over the whole GUI
 	 * tree every time a hotkey is pressed).
-	 * Currently this is only set at load time - dynamic changes to an
-	 * object's hotkey property will be ignored.
 	 */
 	std::map<CStr, std::vector<IGUIObject*> > m_HotkeyObjects;
 
+	/**
+	 * Map from hotkey names to maps of eventNames to functions that are triggered
+	 * when the hotkey goes through the event. Contrary to object hotkeys, this
+	 * allows for only one global function per hotkey name per event type.
+	 */
+	std::map<CStr, std::map<CStr, JS::PersistentRootedValue>> m_GlobalHotkeys;
+
+	/**
+	 * XML and JS can subscribe handlers to events identified by these names.
+	 * Store in static const members to avoid string copies, gain compile errors when misspelling and
+	 * to allow reuse in other classes.
+	 */
+	static const CStr EventNameLoad;
+	static const CStr EventNameTick;
+	static const CStr EventNamePress;
+	static const CStr EventNameKeyDown;
+	static const CStr EventNameRelease;
+	static const CStr EventNameMouseRightPress;
+	static const CStr EventNameMouseLeftPress;
+	static const CStr EventNameMouseWheelDown;
+	static const CStr EventNameMouseWheelUp;
+	static const CStr EventNameMouseWheelLeft;
+	static const CStr EventNameMouseWheelRight;
+	static const CStr EventNameMouseLeftDoubleClick;
+	static const CStr EventNameMouseLeftRelease;
+	static const CStr EventNameMouseRightDoubleClick;
+	static const CStr EventNameMouseRightRelease;
+
 	//--------------------------------------------------------
 	//	Databases
+	//	These are loaded from XML files and marked as noncopyable and const to
+	//	rule out unintentional modification and copy, especially during Draw calls.
 	//--------------------------------------------------------
 
+	// Colors
+	std::map<CStr, const CGUIColor> m_PreDefinedColors;
+
 	// Sprites
-	std::map<CStr, CGUISprite> m_Sprites;
+	std::map<CStr, std::unique_ptr<const CGUISprite>> m_Sprites;
 
 	// Styles
-	std::map<CStr, SGUIStyle> m_Styles;
+	std::map<CStr, const SGUIStyle> m_Styles;
 
 	// Scroll-bar styles
-	std::map<CStr, SGUIScrollBarStyle> m_ScrollBarStyles;
+	std::map<CStr, const SGUIScrollBarStyle> m_ScrollBarStyles;
 
 	// Icons
-	std::map<CStr, SGUIIcon> m_Icons;
+	std::map<CStr, const SGUIIcon> m_Icons;
+
+public:
+	/**
+	 * Map from event names to object which listen to a given event.
+	 */
+	std::unordered_map<CStr, std::vector<IGUIObject*>> m_EventObjects;
 };
 
-#endif
+#endif // INCLUDED_CGUI
