@@ -1,4 +1,4 @@
-/* Copyright (c) 2010 Wildfire Games
+/* Copyright (C) 2020 Wildfire Games.
  *
  * Permission is hereby granted, free of charge, to any person obtaining
  * a copy of this software and associated documentation files (the
@@ -7,10 +7,10 @@
  * distribute, sublicense, and/or sell copies of the Software, and to
  * permit persons to whom the Software is furnished to do so, subject to
  * the following conditions:
- * 
+ *
  * The above copyright notice and this permission notice shall be included
  * in all copies or substantial portions of the Software.
- * 
+ *
  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
  * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
  * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
@@ -27,13 +27,14 @@
 #include "precompiled.h"
 #include "lib/debug_stl.h"
 
+#include "lib/regex.h"
+
+#include <cassert>
 #include <deque>
+#include <list>
 #include <map>
 #include <set>
-#include <cassert>
-#include <list>
-
-#include "lib/regex.h"
+#include <vector>
 
 static const StatusDefinition debugStlStatusDefinitions[] = {
 	{ ERR::STL_CNT_UNKNOWN, L"Unknown STL container type_name" },
@@ -521,7 +522,7 @@ template<class T> bool get_container_info(const T& t, size_t size, size_t el_siz
 {
 	typedef typename T::iterator iterator;
 	typedef typename T::const_iterator const_iterator;
-	
+
 	ENSURE(sizeof(T) == size);
 	ENSURE(sizeof(iterator) < DEBUG_STL_MAX_ITERATOR_SIZE);
 
@@ -549,7 +550,7 @@ template<class T> bool get_container_info(const T& t, size_t size, size_t el_siz
 Status debug_stl_get_container_info(const wchar_t* type_name, const u8* p, size_t size,
 	size_t el_size, size_t* el_count, DebugStlIterator* el_iterator, void* it_mem)
 {
-#if MSC_VERSION >= 1400
+#if MSC_VERSION
 	UNUSED2(type_name);
 	UNUSED2(p);
 	UNUSED2(size);
