@@ -1,4 +1,4 @@
-/* Copyright (c) 2013 Wildfire Games
+/* Copyright (C) 2024 Wildfire Games.
  *
  * Permission is hereby granted, free of charge, to any person obtaining
  * a copy of this software and associated documentation files (the
@@ -7,10 +7,10 @@
  * distribute, sublicense, and/or sell copies of the Software, and to
  * permit persons to whom the Software is furnished to do so, subject to
  * the following conditions:
- * 
+ *
  * The above copyright notice and this permission notice shall be included
  * in all copies or substantial portions of the Software.
- * 
+ *
  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
  * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
  * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
@@ -33,24 +33,13 @@
 
 #include "lib/byte_order.h"	// movzx_le64
 #include "lib/module_init.h"
-#include "lib/sysdep/cpu.h"
 #include "lib/debug_stl.h"
 #include "lib/app_hooks.h"
 #include "lib/external_libraries/dbghelp.h"
 #include "lib/sysdep/os/win/wdbg.h"
 #include "lib/sysdep/os/win/wutil.h"
-#include "lib/sysdep/os/win/winit.h"
-
-WINIT_REGISTER_CRITICAL_INIT(wdbg_sym_Init);
-
-static WUTIL_FUNC(pRtlCaptureContext, VOID, (PCONTEXT));
-
-static Status wdbg_sym_Init()
-{
-	WUTIL_IMPORT_KERNEL32(RtlCaptureContext, pRtlCaptureContext);
-	return INFO::OK;
-}
 
+#include <atomic>
 
 //----------------------------------------------------------------------------
 // dbghelp
@@ -106,7 +95,7 @@ static Status InitDbghelp()
 // symserv wants to access the internet.
 static void sym_init()
 {
-	static ModuleInitState initState;
+	static ModuleInitState initState{ 0 };
 	ModuleInit(&initState, InitDbghelp);
 }
 
@@ -210,8 +199,10 @@ static Status ResolveSymbol_lk(void* ptr_of_interest, wchar_t* sym_name, wchar_t
 	// get source file and/or line number (if requested)
 	if(file || line)
 	{
-		file[0] = '\0';
-		*line = 0;
+		if (file)
+			file[0] = '\0';
+		if (line)
+			*line = 0;
 
 		IMAGEHLP_LINEW64 line_info = { sizeof(IMAGEHLP_LINEW64) };
 		DWORD displacement; // unused but required by pSymGetLineFromAddr64!
@@ -269,14 +260,11 @@ Status debug_CaptureContext(void* pcontext)
 	//   note: it used to be common practice to query the current thread
 	//   context, but WinXP SP2 and above require it be suspended.
 
-	if(!pRtlCaptureContext)
-		return ERR::NOT_SUPPORTED;	// NOWARN
-
 	CONTEXT* context = (CONTEXT*)pcontext;
 	cassert(sizeof(CONTEXT) <= DEBUG_CONTEXT_SIZE);
 	memset(context, 0, sizeof(CONTEXT));
 	context->ContextFlags = CONTEXT_FULL;
-	pRtlCaptureContext(context);
+	RtlCaptureContext(context);
 	return INFO::OK;
 }
 
@@ -313,7 +301,7 @@ Status wdbg_sym_WalkStack(StackFrameCallback cb, uintptr_t cbData, CONTEXT& cont
 	sym_init();
 
 	STACKFRAME64 sf = PopulateStackFrame(context);
-	
+
 	wchar_t func[DEBUG_SYMBOL_CHARS];
 
 	Status ret = ERR::SYM_NO_STACK_FRAMES_FOUND;
@@ -812,7 +800,7 @@ static Status DetermineSymbolAddress(DWORD id, const SYMBOL_INFOW* sym, const Du
 
 	*pp = (const u8*)(uintptr_t)addr;
 
-	debug_printf(L"SYM| %ls at %p  flags=%X dk=%d sym->addr=%I64X fp=%I64x\n", sym->Name, *pp, sym->Flags, dataKind, sym->Address, state.stackFrame->AddrFrame.Offset);
+	debug_printf("SYM| %s at %p  flags=%X dk=%d sym->addr=%I64X fp=%I64x\n", utf8_from_wstring(sym->Name).c_str(), *pp, sym->Flags, dataKind, sym->Address, state.stackFrame->AddrFrame.Offset);
 	return INFO::OK;
 }
 
@@ -827,7 +815,7 @@ static Status DetermineSymbolAddress(DWORD id, const SYMBOL_INFOW* sym, const Du
 // called by dump_sym; lock is held.
 
 static Status dump_sym_array(DWORD type_id, const u8* p, DumpState& state)
-{ 
+{
 	ULONG64 size64 = 0;
 	if(!pSymGetTypeInfo(hProcess, state.moduleBase, type_id, TI_GET_LENGTH, &size64))
 		WARN_RETURN(ERR::SYM_TYPE_INFO_UNAVAILABLE);
@@ -846,7 +834,7 @@ static Status dump_sym_array(DWORD type_id, const u8* p, DumpState& state)
 	ENSURE(el_size != 0);
 	const size_t num_elements = size/el_size;
 	ENSURE(num_elements != 0);
- 
+
 	return dump_array(p, num_elements, el_type_id, el_size, state);
 }
 
@@ -883,7 +871,7 @@ static Status dump_sym_base_type(DWORD type_id, const u8* p, DumpState& state)
 	const wchar_t* fmt = L"";
 
 	u64 data = movzx_le64(p, size);
-	// if value is 0xCC..CC (uninitialized mem), we display as hex. 
+	// if value is 0xCC..CC (uninitialized mem), we display as hex.
 	// the output would otherwise be garbage; this makes it obvious.
 	// note: be very careful to correctly handle size=0 (e.g. void*).
 	for(size_t i = 0; i < size; i++)
@@ -1170,7 +1158,7 @@ static bool ptr_already_visited(const u8* p)
 		static bool haveComplained;
 		if(!haveComplained)
 		{
-			debug_printf(L"WARNING: ptr_already_visited: capacity exceeded, increase maxVisited\n");
+			debug_printf("WARNING: ptr_already_visited: capacity exceeded, increase maxVisited\n");
 			debug_break();
 			haveComplained = true;
 		}
@@ -1443,7 +1431,7 @@ static bool udt_fits_on_one_line(const wchar_t* type_name, size_t child_count, s
 static Status udt_dump_normal(const wchar_t* type_name, const u8* p, size_t size, DumpState state, ULONG numChildren, const DWORD* children)
 {
 	// special case: boost::unordered types are complex and may cause a stack overflow
-	// see http://trac.wildfiregames.com/ticket/1813
+	// see https://gitea.wildfiregames.com/0ad/0ad/issues/1813
 	// TODO: at least give some info about them
 	if(!wcsncmp(type_name, L"boost::unordered", 16))
 		return INFO::CANNOT_HANDLE;
@@ -1469,7 +1457,7 @@ static Status udt_dump_normal(const wchar_t* type_name, const u8* p, size_t size
 			continue;
 		if(ofs >= size)
 		{
-			debug_printf(L"INVALID_UDT %ls %d %d\n", type_name, ofs, size);
+			debug_printf("INVALID_UDT %s %d %d\n", utf8_from_wstring(type_name).c_str(), ofs, size);
 		}
 		//ENSURE(ofs < size);
 
@@ -1581,7 +1569,7 @@ static Status dump_sym_unknown(DWORD type_id, const u8* UNUSED(p), DumpState& st
 	if(!pSymGetTypeInfo(hProcess, state.moduleBase, type_id, TI_GET_SYMTAG, &type_tag))
 		WARN_RETURN(ERR::SYM_TYPE_INFO_UNAVAILABLE);
 
-	debug_printf(L"SYM: unknown tag: %d\n", type_tag);
+	debug_printf("SYM: unknown tag: %d\n", type_tag);
 	out(L"(unknown symbol type)");
 	return INFO::OK;
 }
@@ -1729,8 +1717,9 @@ static Status dump_frame_cb(const STACKFRAME64* sf, uintptr_t UNUSED(userContext
 
 Status debug_DumpStack(wchar_t* buf, size_t maxChars, void* pcontext, const wchar_t* lastFuncToSkip)
 {
-	static intptr_t busy;
-	if(!cpu_CAS(&busy, 0, 1))
+	static std::atomic<bool> busy{ false };
+
+	if(busy.exchange(true))
 		return ERR::REENTERED;	// NOWARN
 
 	out_init(buf, maxChars);
@@ -1739,8 +1728,7 @@ Status debug_DumpStack(wchar_t* buf, size_t maxChars, void* pcontext, const wcha
 	wdbg_assert(pcontext != 0);
 	Status ret = wdbg_sym_WalkStack(dump_frame_cb, 0, *(CONTEXT*)pcontext, lastFuncToSkip);
 
-	COMPILER_FENCE;
-	busy = 0;
+	busy = false;
 
 	return ret;
 }
@@ -1776,9 +1764,8 @@ void wdbg_sym_WriteMinidump(EXCEPTION_POINTERS* exception_pointers)
 	// (UserStreamParam), since we will need to generate a plain text file on
 	// non-Windows platforms. users will just have to send us both files.
 
-	HANDLE hProcess = GetCurrentProcess();
 	DWORD pid = GetCurrentProcessId();
-	if(!pMiniDumpWriteDump || !pMiniDumpWriteDump(hProcess, pid, hFile, MiniDumpNormal, &mei, 0, 0))
+	if(!pMiniDumpWriteDump || !pMiniDumpWriteDump(GetCurrentProcess(), pid, hFile, MiniDumpNormal, &mei, 0, 0))
 		DEBUG_DISPLAY_ERROR(L"wdbg_sym_WriteMinidump: unable to generate minidump.");
 
 	CloseHandle(hFile);
