@@ -1,4 +1,4 @@
-/* Copyright (c) 2010 Wildfire Games
+/* Copyright (C) 2022 Wildfire Games.
  *
  * Permission is hereby granted, free of charge, to any person obtaining
  * a copy of this software and associated documentation files (the
@@ -7,10 +7,10 @@
  * distribute, sublicense, and/or sell copies of the Software, and to
  * permit persons to whom the Software is furnished to do so, subject to
  * the following conditions:
- * 
+ *
  * The above copyright notice and this permission notice shall be included
  * in all copies or substantial portions of the Software.
- * 
+ *
  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
  * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
  * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
@@ -51,7 +51,7 @@ class S3tcBlock
 {
 public:
 	S3tcBlock(size_t dxt, const u8* RESTRICT block)
-		: dxt(dxt)
+		: m_Dxt(dxt)
 	{
 		// (careful, 'dxt != 1' doesn't work - there's also DXT1a)
 		const u8* a_block = block;
@@ -71,23 +71,23 @@ public:
 			out[i] = (u8)c[c_selector][i];
 
 		// if no alpha, done
-		if(dxt == 1)
+		if(m_Dxt == 1)
 			return;
 
 		size_t a;
-		if(dxt == 3)
+		if(m_Dxt == 3)
 		{
 			// table of 4-bit alpha entries
 			a = access_bit_tbl(a_bits, pixel_idx, 4);
 			a |= a << 4; // expand to 8 bits (replicate high into low!)
 		}
-		else if(dxt == 5)
+		else if(m_Dxt == 5)
 		{
 			// pixel index -> alpha selector (3 bit) -> alpha
 			const size_t a_selector = access_bit_tbl(a_bits, pixel_idx, 3);
 			a = dxt5_a_tbl[a_selector];
 		}
-		// (dxt == DXT1A)
+		// (m_Dxt == DXT1A)
 		else
 			a = c[c_selector][A];
 		out[A] = (u8)(a & 0xFF);
@@ -214,7 +214,7 @@ private:
 	// table of 2-bit color selectors
 	u32 c_selectors;
 
-	size_t dxt;
+	size_t m_Dxt;
 };
 
 
@@ -279,22 +279,22 @@ static Status s3tc_decompress(Tex* t)
 	// - adding or stripping alpha channels during transform is not
 	//   our job; we merely output the same pixel format as given
 	//   (tex.cpp's plain transform could cover it, if ever needed).
-	const size_t dxt = t->flags & TEX_DXT;
+	const size_t dxt = t->m_Flags & TEX_DXT;
 	const size_t out_bpp = (dxt != 1)? 32 : 24;
-	const size_t out_size = tex_img_size(t) * out_bpp / t->bpp;
-	shared_ptr<u8> decompressedData;
-	AllocateAligned(decompressedData, out_size, pageSize);
+	const size_t out_size = t->img_size() * out_bpp / t->m_Bpp;
+	std::shared_ptr<u8> decompressedData;
+	AllocateAligned(decompressedData, out_size, g_PageSize);
 
 	const size_t s3tc_block_size = (dxt == 3 || dxt == 5)? 16 : 8;
 	S3tcDecompressInfo di = { dxt, s3tc_block_size, out_bpp/8, decompressedData.get() };
-	const u8* s3tc_data = tex_get_data(t);
-	const int levels_to_skip = (t->flags & TEX_MIPMAPS)? 0 : TEX_BASE_LEVEL_ONLY;
-	tex_util_foreach_mipmap(t->w, t->h, t->bpp, s3tc_data, levels_to_skip, 4, s3tc_decompress_level, &di);
-	t->data = decompressedData;
-	t->dataSize = out_size;
-	t->ofs = 0;
-	t->bpp = out_bpp;
-	t->flags &= ~TEX_DXT;
+	const u8* s3tc_data = t->get_data();
+	const int levels_to_skip = (t->m_Flags & TEX_MIPMAPS)? 0 : TEX_BASE_LEVEL_ONLY;
+	tex_util_foreach_mipmap(t->m_Width, t->m_Height, t->m_Bpp, s3tc_data, levels_to_skip, 4, s3tc_decompress_level, &di);
+	t->m_Data = decompressedData;
+	t->m_DataSize = out_size;
+	t->m_Ofs = 0;
+	t->m_Bpp = out_bpp;
+	t->m_Flags &= ~TEX_DXT;
 	return INFO::OK;
 }
 
@@ -303,17 +303,20 @@ static Status s3tc_decompress(Tex* t)
 // DDS file format
 //-----------------------------------------------------------------------------
 
-// bit values and structure definitions taken from 
-// http://msdn.microsoft.com/en-us/library/ee417785(VS.85).aspx
+// bit values and structure definitions taken from
+// https://docs.microsoft.com/en-us/windows/win32/direct3ddds/dx-graphics-dds-reference
 
 #pragma pack(push, 1)
 
 // DDS_PIXELFORMAT.dwFlags
+
+// This is used to distinguish RGBA from RGB and DXT1a from DXT1.
 // we've seen some DXT3 files that don't have this set (which is nonsense;
-// any image lacking alpha should be stored as DXT1). it's authoritative
-// if fourcc is DXT1 (there's no other way to tell DXT1 and DXT1a apart)
-// and ignored otherwise.
+// any image lacking alpha should be stored as DXT1).
 #define DDPF_ALPHAPIXELS 0x00000001
+// DDPF_ALPHA is used for uncompressed 8bpp greyscale, in which the data
+// is stored in the alpha mask.
+#define DDPF_ALPHA       0x00000002
 #define DDPF_FOURCC      0x00000004
 #define DDPF_RGB         0x00000040
 
@@ -326,7 +329,7 @@ struct DDS_PIXELFORMAT
 	u32 dwRBitMask;
 	u32 dwGBitMask;
 	u32 dwBBitMask;
-	u32 dwABitMask;                   // (DDPF_ALPHAPIXELS)
+	u32 dwABitMask;                   // (DDPF_ALPHA or DDPF_ALPHAPIXELS)
 };
 
 
@@ -394,7 +397,7 @@ static Status decode_pf(const DDS_PIXELFORMAT* pf, size_t& bpp, size_t& flags)
 
 	// check struct size
 	if(read_le32(&pf->dwSize) != sizeof(DDS_PIXELFORMAT))
-		WARN_RETURN(ERR::TEX_INVALID_SIZE);
+		return ERR::TEX_INVALID_SIZE;
 
 	// determine type
 	const size_t pf_flags = (size_t)read_le32(&pf->dwFlags);
@@ -429,13 +432,13 @@ static Status decode_pf(const DDS_PIXELFORMAT* pf, size_t& bpp, size_t& flags)
 			// DDS is storing images in a format that requires no processing,
 			// we do not allow any weird orderings that require runtime work.
 			// instead, the artists must export with the correct settings.
-			WARN_RETURN(ERR::TEX_FMT_INVALID);
+			return ERR::TEX_FMT_INVALID;
 		}
 
 		RETURN_STATUS_IF_ERR(tex_validate_plain_format(bpp, (int)flags));
 	}
 	// .. uncompressed 8bpp greyscale
-	else if(pf_flags & DDPF_ALPHAPIXELS)
+	else if(pf_flags & DDPF_ALPHA)
 	{
 		const size_t pf_bpp    = (size_t)read_le32(&pf->dwRGBBitCount);
 		const size_t pf_a_mask = (size_t)read_le32(&pf->dwABitMask);
@@ -443,10 +446,10 @@ static Status decode_pf(const DDS_PIXELFORMAT* pf, size_t& bpp, size_t& flags)
 		bpp = pf_bpp;
 
 		if(pf_bpp != 8)
-			WARN_RETURN(ERR::TEX_FMT_INVALID);
+			return ERR::TEX_FMT_INVALID;
 
 		if(pf_a_mask != 0xFF)
-			WARN_RETURN(ERR::TEX_FMT_INVALID);
+			return ERR::TEX_FMT_INVALID;
 		flags |= TEX_GREY;
 
 		RETURN_STATUS_IF_ERR(tex_validate_plain_format(bpp, (int)flags));
@@ -477,12 +480,12 @@ static Status decode_pf(const DDS_PIXELFORMAT* pf, size_t& bpp, size_t& flags)
 			break;
 
 		default:
-			WARN_RETURN(ERR::TEX_FMT_INVALID);
+			return ERR::TEX_FMT_INVALID;
 		}
 	}
 	// .. neither uncompressed nor compressed - invalid
 	else
-		WARN_RETURN(ERR::TEX_FMT_INVALID);
+		return ERR::TEX_FMT_INVALID;
 
 	return INFO::OK;
 }
@@ -496,7 +499,7 @@ static Status decode_sd(const DDS_HEADER* sd, size_t& w, size_t& h, size_t& bpp,
 {
 	// check header size
 	if(read_le32(&sd->dwSize) != sizeof(*sd))
-		WARN_RETURN(ERR::CORRUPTED);
+		return ERR::CORRUPTED;
 
 	// flags (indicate which fields are valid)
 	const size_t sd_flags = (size_t)read_le32(&sd->dwFlags);
@@ -504,7 +507,7 @@ static Status decode_sd(const DDS_HEADER* sd, size_t& w, size_t& h, size_t& bpp,
 	// note: we can't guess dimensions - the image may not be square.
 	const size_t sd_req_flags = DDSD_CAPS|DDSD_HEIGHT|DDSD_WIDTH|DDSD_PIXELFORMAT;
 	if((sd_flags & sd_req_flags) != sd_req_flags)
-		WARN_RETURN(ERR::TEX_INCOMPLETE_HEADER);
+		return ERR::TEX_INCOMPLETE_HEADER;
 
 	// image dimensions
 	h = (size_t)read_le32(&sd->dwHeight);
@@ -534,15 +537,15 @@ static Status decode_sd(const DDS_HEADER* sd, size_t& w, size_t& h, size_t& bpp,
 	if(sd_flags & DDSD_PITCH)
 	{
 		if(sd_pitch_or_size != Align<4>(pitch))
-			DEBUG_WARN_ERR(ERR::CORRUPTED);
+			return ERR::CORRUPTED;
 	}
 	if(sd_flags & DDSD_LINEARSIZE)
 	{
 		// some DDS tools mistakenly store the total size of all levels,
 		// so allow values close to that as well
 		const ssize_t totalSize = ssize_t(pitch*stored_h*1.333333f);
-		if(sd_pitch_or_size != pitch*stored_h && abs(ssize_t(sd_pitch_or_size)-totalSize) > 64)
-			DEBUG_WARN_ERR(ERR::CORRUPTED);
+		if(sd_pitch_or_size != pitch*stored_h && std::abs(ssize_t(sd_pitch_or_size)-totalSize) > 64)
+			return ERR::CORRUPTED;
 	}
 	// note: both flags set would be invalid; no need to check for that,
 	// though, since one of the above tests would fail.
@@ -556,7 +559,7 @@ static Status decode_sd(const DDS_HEADER* sd, size_t& w, size_t& h, size_t& bpp,
 			// mipmap chain is incomplete
 			// note: DDS includes the base level in its count, hence +1.
 			if(mipmap_count != ceil_log2(std::max(w,h))+1)
-				WARN_RETURN(ERR::TEX_FMT_INVALID);
+				return ERR::TEX_FMT_INVALID;
 			flags |= TEX_MIPMAPS;
 		}
 	}
@@ -566,17 +569,19 @@ static Status decode_sd(const DDS_HEADER* sd, size_t& w, size_t& h, size_t& bpp,
 	{
 		const size_t depth = (size_t)read_le32(&sd->dwDepth);
 		if(depth)
-			WARN_RETURN(ERR::NOT_SUPPORTED);
+			return ERR::NOT_SUPPORTED;
 	}
 
 	// check caps
 	// .. this is supposed to be set, but don't bail if not (pointless)
-	ENSURE(sd->dwCaps & DDSCAPS_TEXTURE);
+	if (!(sd->dwCaps & DDSCAPS_TEXTURE))
+		return ERR::CORRUPTED;
 	// .. sanity check: warn if mipmap flag not set (don't bail if not
 	// because we've already made the decision).
 	const bool mipmap_cap = (sd->dwCaps & DDSCAPS_MIPMAP) != 0;
 	const bool mipmap_flag = (flags & TEX_MIPMAPS) != 0;
-	ENSURE(mipmap_cap == mipmap_flag);
+	if (mipmap_cap != mipmap_flag)
+		return ERR::CORRUPTED;
 	// note: we do not check for cubemaps and volume textures (not supported)
 	// because the file may still have useful data we can read.
 
@@ -586,33 +591,33 @@ static Status decode_sd(const DDS_HEADER* sd, size_t& w, size_t& h, size_t& bpp,
 
 //-----------------------------------------------------------------------------
 
-static bool dds_is_hdr(const u8* file)
+bool TexCodecDds::is_hdr(const u8* file) const
 {
 	return *(u32*)file == FOURCC('D','D','S',' ');
 }
 
 
-static bool dds_is_ext(const OsPath& extension)
+bool TexCodecDds::is_ext(const OsPath& extension) const
 {
 	return extension == L".dds";
 }
 
 
-static size_t dds_hdr_size(const u8* UNUSED(file))
+size_t TexCodecDds::hdr_size(const u8* UNUSED(file)) const
 {
 	return 4+sizeof(DDS_HEADER);
 }
 
 
-static Status dds_decode(rpU8 data, size_t UNUSED(size), Tex* RESTRICT t)
+Status TexCodecDds::decode(u8* RESTRICT data, size_t UNUSED(size), Tex* RESTRICT t) const
 {
 	const DDS_HEADER* sd = (const DDS_HEADER*)(data+4);
-	RETURN_STATUS_IF_ERR(decode_sd(sd, t->w, t->h, t->bpp, t->flags));
+	RETURN_STATUS_IF_ERR(decode_sd(sd, t->m_Width, t->m_Height, t->m_Bpp, t->m_Flags));
 	return INFO::OK;
 }
 
 
-static Status dds_encode(Tex* RESTRICT UNUSED(t), DynArray* RESTRICT UNUSED(da))
+Status TexCodecDds::encode(Tex* RESTRICT UNUSED(t), DynArray* RESTRICT UNUSED(da)) const
 {
 	// note: do not return ERR::NOT_SUPPORTED et al. because that would
 	// break tex_write (which assumes either this, 0 or errors are returned).
@@ -622,12 +627,12 @@ static Status dds_encode(Tex* RESTRICT UNUSED(t), DynArray* RESTRICT UNUSED(da))
 
 TIMER_ADD_CLIENT(tc_dds_transform);
 
-static Status dds_transform(Tex* t, size_t transforms)
+Status TexCodecDds::transform(Tex* t, size_t transforms) const
 {
 	TIMER_ACCRUE(tc_dds_transform);
 
-	size_t mipmaps = t->flags & TEX_MIPMAPS;
-	size_t dxt = t->flags & TEX_DXT;
+	size_t mipmaps = t->m_Flags & TEX_MIPMAPS;
+	size_t dxt = t->m_Flags & TEX_DXT;
 	ENSURE(is_valid_dxt(dxt));
 
 	const size_t transform_mipmaps = transforms & TEX_MIPMAPS;
@@ -637,7 +642,7 @@ static Status dds_transform(Tex* t, size_t transforms)
 	{
 		// we don't need to actually change anything except the flag - the
 		// mipmap levels will just be treated as trailing junk
-		t->flags &= ~TEX_MIPMAPS;
+		t->m_Flags &= ~TEX_MIPMAPS;
 		return INFO::OK;
 	}
 	// requesting decompression
@@ -651,6 +656,3 @@ static Status dds_transform(Tex* t, size_t transforms)
 	// both not DXT (nothing we can do) - bail.
 	return INFO::TEX_CODEC_CANNOT_HANDLE;
 }
-
-
-TEX_CODEC_REGISTER(dds);
