@@ -169,6 +169,8 @@ XmppClient::XmppClient(const ScriptInterface* scriptInterface, const std::string
  */
 XmppClient::~XmppClient()
 {
+	this->disconnect();
+
 	DbgXMPP("XmppClient destroyed");
 	delete m_registration;
 	delete m_mucRoom;
@@ -1183,9 +1185,9 @@ const std::wstring& XmppClient::GetSubject()
 }
 
 /**
- * Request nick change, real change via mucRoomHandler.
+ * Request MUC nick change, real change via mucRoomHandler.
  *
- * @param nick Desired nickname
+ * @param nick Desired MUC nickname
  */
 void XmppClient::SetNick(const std::string& nick)
 {
@@ -1193,7 +1195,7 @@ void XmppClient::SetNick(const std::string& nick)
 }
 
 /**
- * Get current nickname.
+ * Get current MUC nickname.
  */
 std::string XmppClient::GetNick() const
 {
@@ -1205,6 +1207,17 @@ std::string XmppClient::GetJID() const
 	return m_client->jid().full();
 }
 
+
+/**
+ * Get the XMPP username.
+ *
+ * @return current XMPP username
+ */
+std::string XmppClient::GetUsername() const
+{
+	return m_username;
+}
+
 /**
  * Change password for authenticated user.
  *
@@ -1212,7 +1225,7 @@ std::string XmppClient::GetJID() const
  */
 void XmppClient::ChangePassword(const std::string& newPassword)
 {
-	m_registration->changePassword(m_username, newPassword);
+	m_registration->changePassword(m_client->jid().username(), newPassword);
 }
 
 /**
