@@ -193,20 +193,20 @@ bool CNetServerWorker::SetupConnection(const u16 port)
 
 #if CONFIG2_MINIUPNPC
 	// Launch the UPnP thread
-	m_UPnPThread = std::thread(Threading::HandleExceptions<SetupUPnP>::Wrapper);
+	m_UPnPThread = std::thread(Threading::HandleExceptions<SetupUPnP>::Wrapper, port);
 #endif
 
 	return true;
 }
 
 #if CONFIG2_MINIUPNPC
-void CNetServerWorker::SetupUPnP()
+void CNetServerWorker::SetupUPnP(const u16 port)
 {
 	debug_SetThreadName("UPnP");
 
 	// Values we want to set.
 	char psPort[6];
-	sprintf_s(psPort, ARRAY_SIZE(psPort), "%d", PS_DEFAULT_PORT);
+	sprintf_s(psPort, ARRAY_SIZE(psPort), "%d", port);
 	const char* leaseDuration = "0"; // Indefinite/permanent lease duration.
 	const char* description = "0AD Multiplayer";
 	const char* protocall = "UDP";
@@ -242,7 +242,7 @@ void CNetServerWorker::SetupUPnP()
 	int ret = 0;
 
 	// Try a cached URL first
-	if (!rootDescURL.empty() && UPNP_GetIGDFromUrl(rootDescURL.c_str(), &urls, &data, internalIPAddress, sizeof(internalIPAddress)))
+	if (!rootDescURL.empty() && UPNP_GetIGDFromUrl(rootDescURL.c_str(), &urls, &data, internalIPAddress, sizeof(internalIPAddress)) && strlen(data.first.controlurl) != 0)
 	{
 		LOGMESSAGE("Net server: using cached IGD = %s", urls.controlURL);
 		ret = 1;
@@ -339,9 +339,9 @@ void CNetServerWorker::SetupUPnP()
 				   externalIPAddress, psPort, protocall, intClient, intPort, duration);
 
 	// Cache root descriptor URL to try to avoid discovery next time.
-	g_ConfigDB.SetValueString(CFG_USER, "network.upnprootdescurl", urls.controlURL);
-	g_ConfigDB.WriteValueToFile(CFG_USER, "network.upnprootdescurl", urls.controlURL);
-	LOGMESSAGE("Net server: cached UPnP root descriptor URL as %s", urls.controlURL);
+	g_ConfigDB.SetValueString(CFG_USER, "network.upnprootdescurl", urls.rootdescURL);
+	g_ConfigDB.WriteValueToFile(CFG_USER, "network.upnprootdescurl", urls.rootdescURL);
+	LOGMESSAGE("Net server: cached UPnP root descriptor URL as %s", urls.rootdescURL);
 
 	freeUPnP();
 }
@@ -424,7 +424,7 @@ bool CNetServerWorker::RunStep()
 	// (Do as little work as possible while the mutex is held open,
 	// to avoid performance problems and deadlocks.)
 
-	m_ScriptInterface->GetContext().MaybeIncrementalGC(0.5f);
+	m_ScriptInterface->GetContext().MaybeIncrementalGC();
 
 	ScriptRequest rq(m_ScriptInterface);
 
