@@ -1,4 +1,4 @@
-/* Copyright (C) 2009 Wildfire Games.
+/* Copyright (C) 2021 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -15,10 +15,16 @@
  * along with 0 A.D.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#define MINIMAL_PCH 1
+#define MINIMAL_PCH 2
 #include "lib/precompiled.h"	// common precompiled header
 
-// Minimal is a bit *too* minimal to let things compile, so include a few more headers
-#include "lib/debug.h"
-#include <boost/shared_ptr.hpp>
-using boost::shared_ptr;
+#if CONFIG_ENABLE_PCH
+
+// The simulation has many header files (component interfaces) which are quick to compile on their own
+// but bring in a lot of headers indirectly. Adding these to the precompiled header
+// cuts down compilation time by about half.
+#include "simulation2/system/Interface.h"
+#include "maths/FixedVector2D.h"
+#include "maths/FixedVector3D.h"
+
+#endif // CONFIG_ENABLE_PCH
