@@ -62,6 +62,8 @@ CBinarySerializerScriptImpl::CBinarySerializerScriptImpl(const ScriptInterface&
 {
 	ScriptRequest rq(m_ScriptInterface);
 	JS_AddExtraGCRootsTracer(rq.cx, Trace, this);
+	m_SerializePropId = JS::PropertyKey::fromPinnedString(JS_AtomizeAndPinString(rq.cx, "Serialize"));
+	m_DeserializePropId = JS::PropertyKey::fromPinnedString(JS_AtomizeAndPinString(rq.cx, "Deserialize"));
 }
 
 CBinarySerializerScriptImpl::~CBinarySerializerScriptImpl()
@@ -70,10 +72,13 @@ CBinarySerializerScriptImpl::~CBinarySerializerScriptImpl()
 	JS_RemoveExtraGCRootsTracer(rq.cx, Trace, this);
 }
 
-void CBinarySerializerScriptImpl::HandleScriptVal(JS::HandleValue val)
+void CBinarySerializerScriptImpl::PutScriptVal(JS::HandleValue val)
 {
-	ScriptRequest rq(m_ScriptInterface);
+	HandleScriptVal(ScriptRequest(m_ScriptInterface), val);
+}
 
+void CBinarySerializerScriptImpl::HandleScriptVal(const ScriptRequest& rq, JS::HandleValue val)
+{
 	switch (JS_TypeOfValue(rq.cx, val))
 	{
 	case JSTYPE_UNDEFINED:
@@ -92,7 +97,7 @@ void CBinarySerializerScriptImpl::HandleScriptVal(JS::HandleValue val)
 		JS::RootedObject obj(rq.cx, &val.toObject());
 
 		// If we've already serialized this object, just output a reference to it
-		u32 tag = GetScriptBackrefTag(obj);
+		u32 tag = GetScriptBackrefTag(rq, obj);
 		if (tag != 0)
 		{
 			m_Serializer.NumberU8_Unbounded("type", SCRIPT_TYPE_BACKREF);
@@ -129,7 +134,7 @@ void CBinarySerializerScriptImpl::HandleScriptVal(JS::HandleValue val)
 			// Now handle its array buffer
 			// this may be a backref, since ArrayBuffers can be shared by multiple views
 			JS::RootedValue bufferVal(rq.cx, JS::ObjectValue(*JS_GetArrayBufferViewBuffer(rq.cx, obj, &sharedMemory)));
-			HandleScriptVal(bufferVal);
+			HandleScriptVal(rq, bufferVal);
 			break;
 		}
 		else if (JS::IsArrayBufferObject(obj))
@@ -177,8 +182,8 @@ void CBinarySerializerScriptImpl::HandleScriptVal(JS::HandleValue val)
 				ENSURE(JS_GetElement(rq.cx, keyValuePairObj, 0, &key));
 				ENSURE(JS_GetElement(rq.cx, keyValuePairObj, 1, &value));
 
-				HandleScriptVal(key);
-				HandleScriptVal(value);
+				HandleScriptVal(rq, key);
+				HandleScriptVal(rq, value);
 			}
 			break;
 		}
@@ -206,7 +211,7 @@ void CBinarySerializerScriptImpl::HandleScriptVal(JS::HandleValue val)
 				if (done)
 					break;
 
-				HandleScriptVal(value);
+				HandleScriptVal(rq, value);
 			}
 			break;
 		}
@@ -227,14 +232,14 @@ void CBinarySerializerScriptImpl::HandleScriptVal(JS::HandleValue val)
 				if (!JS_GetPrototype(rq.cx, obj, &proto))
 					throw PSERROR_Serialize_ScriptError("JS_GetPrototype failed");
 
-				SPrototypeSerialization protoInfo = GetPrototypeInfo(rq, proto);
+				SPrototypeSerialization protoInfo = GetPrototypeInfo(rq, proto, m_SerializePropId, m_DeserializePropId);
 
-				if (protoInfo.name == "Object")
+				if (protoInfo.name == L"Object")
 					m_Serializer.NumberU8_Unbounded("type", SCRIPT_TYPE_OBJECT);
 				else
 				{
 					m_Serializer.NumberU8_Unbounded("type", SCRIPT_TYPE_OBJECT_PROTOTYPE);
-					m_Serializer.String("proto", wstring_from_utf8(protoInfo.name), 0, 256);
+					m_Serializer.String("proto", protoInfo.name, 0, 256);
 
 					// Does it have custom Serialize function?
 					// if so, we serialize the data it returns, rather than the object's properties directly
@@ -281,7 +286,7 @@ void CBinarySerializerScriptImpl::HandleScriptVal(JS::HandleValue val)
 				JS::RootedString str(rq.cx, JS::ToString(rq.cx, val));
 				if (!str)
 					throw PSERROR_Serialize_ScriptError("JS_ValueToString failed");
-				ScriptString("value", str);
+				ScriptString(rq, "value", str);
 				break;
 			}
 			else if (protokey == JSProto_Boolean)
@@ -330,12 +335,12 @@ void CBinarySerializerScriptImpl::HandleScriptVal(JS::HandleValue val)
 			if (!idstr)
 				throw PSERROR_Serialize_ScriptError("JS_ValueToString failed");
 
-			ScriptString("prop name", idstr);
+			ScriptString(rq, "prop name", idstr);
 
 			if (!JS_GetPropertyById(rq.cx, obj, id, &propval))
 				throw PSERROR_Serialize_ScriptError("JS_GetPropertyById failed");
 
-			HandleScriptVal(propval);
+			HandleScriptVal(rq, propval);
 		}
 
 		break;
@@ -376,7 +381,7 @@ void CBinarySerializerScriptImpl::HandleScriptVal(JS::HandleValue val)
 	{
 		m_Serializer.NumberU8_Unbounded("type", SCRIPT_TYPE_STRING);
 		JS::RootedString stringVal(rq.cx, val.toString());
-		ScriptString("string", stringVal);
+		ScriptString(rq, "string", stringVal);
 		break;
 	}
 	case JSTYPE_NUMBER:
@@ -425,10 +430,8 @@ void CBinarySerializerScriptImpl::HandleScriptVal(JS::HandleValue val)
 	}
 }
 
-void CBinarySerializerScriptImpl::ScriptString(const char* name, JS::HandleString string)
+void CBinarySerializerScriptImpl::ScriptString(const ScriptRequest& rq, const char* name, JS::HandleString string)
 {
-	ScriptRequest rq(m_ScriptInterface);
-
 #if BYTE_ORDER != LITTLE_ENDIAN
 #error TODO: probably need to convert JS strings to little-endian
 #endif
@@ -436,8 +439,8 @@ void CBinarySerializerScriptImpl::ScriptString(const char* name, JS::HandleStrin
 	size_t length;
 	JS::AutoCheckCannotGC nogc;
 	// Serialize strings directly as UTF-16 or Latin1, to avoid expensive encoding conversions
-	bool isLatin1 = JS::StringHasLatin1Chars(string);
-	m_Serializer.Bool("isLatin1", isLatin1);
+	u8 isLatin1 = JS::StringHasLatin1Chars(string);
+	m_Serializer.NumberU8_Unbounded("isLatin1", isLatin1);
 	if (isLatin1)
 	{
 		const JS::Latin1Char* chars = JS_GetLatin1StringCharsAndLength(rq.cx, nogc, string, &length);
@@ -463,15 +466,13 @@ void CBinarySerializerScriptImpl::Trace(JSTracer *trc, void *data)
 	serializer->m_ScriptBackrefTags.trace(trc);
 }
 
-u32 CBinarySerializerScriptImpl::GetScriptBackrefTag(JS::HandleObject obj)
+u32 CBinarySerializerScriptImpl::GetScriptBackrefTag(const ScriptRequest& rq, JS::HandleObject obj)
 {
 	// To support non-tree structures (e.g. "var x = []; var y = [x, x];"), we need a way
 	// to indicate multiple references to one object(/array). So every time we serialize a
 	// new object, we give it a new tag; when we serialize it a second time we just refer
 	// to that tag.
 
-	ScriptRequest rq(m_ScriptInterface);
-
 	ObjectTagMap::Ptr ptr = m_ScriptBackrefTags.lookup(JS::Heap<JSObject*>(obj.get()));
 	if (!ptr.found())
 	{
