@@ -1,4 +1,4 @@
-/* Copyright (C) 2013 Wildfire Games.
+/* Copyright (C) 2023 Wildfire Games.
  * This file is part of 0 A.D.
  *
  * 0 A.D. is free software: you can redistribute it and/or modify
@@ -32,7 +32,7 @@
 #include "graphics/ObjectManager.h"
 #include "graphics/Terrain.h"
 #include "graphics/Unit.h"
-#include "lib/ogl.h"
+#include "lib/utf8.h"
 #include "maths/MathUtil.h"
 #include "maths/Matrix3D.h"
 #include "ps/CLogger.h"
@@ -41,6 +41,7 @@
 #include "renderer/Renderer.h"
 #include "renderer/WaterManager.h"
 #include "simulation2/Simulation2.h"
+#include "simulation2/components/ICmpObstruction.h"
 #include "simulation2/components/ICmpOwnership.h"
 #include "simulation2/components/ICmpPosition.h"
 #include "simulation2/components/ICmpPlayer.h"
@@ -49,9 +50,10 @@
 #include "simulation2/components/ICmpTemplateManager.h"
 #include "simulation2/components/ICmpVisual.h"
 #include "simulation2/helpers/Selection.h"
+#include "ps/XML/XMLWriter.h"
 
-
-namespace AtlasMessage {
+namespace AtlasMessage
+{
 
 namespace
 {
@@ -59,6 +61,31 @@ namespace
 	{
 		return wcscmp(a.name.c_str(), b.name.c_str()) < 0;
 	}
+} // anonymous namespace
+
+// Helpers for object constraints
+bool CheckEntityObstruction(entity_id_t ent)
+{
+	CmpPtr<ICmpObstruction> cmpObstruction(*g_Game->GetSimulation2(), ent);
+	if (cmpObstruction)
+	{
+		ICmpObstruction::EFoundationCheck result = cmpObstruction->CheckFoundation("default");
+		if (result != ICmpObstruction::FOUNDATION_CHECK_SUCCESS)
+			return false;
+	}
+	return true;
+}
+
+void CheckObstructionAndUpdateVisual(entity_id_t id)
+{
+	CmpPtr<ICmpVisual> cmpVisual(*g_Game->GetSimulation2(), id);
+	if (cmpVisual)
+	{
+		if (!CheckEntityObstruction(id))
+			cmpVisual->SetShadingColor(fixed::FromDouble(1.4), fixed::FromDouble(0.4), fixed::FromDouble(0.4), fixed::FromDouble(1));
+		else
+			cmpVisual->SetShadingColor(fixed::FromDouble(1), fixed::FromDouble(1), fixed::FromDouble(1), fixed::FromDouble(1));
+	}
 }
 
 QUERYHANDLER(GetObjectsList)
@@ -101,7 +128,7 @@ typedef std::map<player_id_t, CColor> PlayerColorMap;
 // Helper function to find color of player owning the given entity,
 //	returns white if entity has no owner. Uses caching to avoid
 //	expensive script calls.
-static CColor GetOwnerPlayerColor(PlayerColorMap& colourMap, entity_id_t id)
+static CColor GetOwnerPlayerColor(PlayerColorMap& colorMap, entity_id_t id)
 {
 	// Default color - white
 	CColor color(1.0f, 1.0f, 1.0f, 1.0f);
@@ -111,15 +138,18 @@ static CColor GetOwnerPlayerColor(PlayerColorMap& colourMap, entity_id_t id)
 	if (cmpOwnership)
 	{
 		player_id_t owner = cmpOwnership->GetOwner();
-		if (colourMap.find(owner) != colourMap.end())
-			return colourMap[owner];
+		if (colorMap.find(owner) != colorMap.end())
+			return colorMap[owner];
 		else
 		{
 			CmpPtr<ICmpPlayerManager> cmpPlayerManager(sim, SYSTEM_ENTITY);
 			entity_id_t playerEnt = cmpPlayerManager->GetPlayerByID(owner);
 			CmpPtr<ICmpPlayer> cmpPlayer(sim, playerEnt);
 			if (cmpPlayer)
-				color = colourMap[owner] = cmpPlayer->GetColour();
+			{
+				colorMap[owner] = cmpPlayer->GetDisplayedColor();
+				color = colorMap[owner];
+			}
 		}
 	}
 	return color;
@@ -129,7 +159,7 @@ MESSAGEHANDLER(SetSelectionPreview)
 {
 	CSimulation2& sim = *g_Game->GetSimulation2();
 
-	// Cache player colours for performance
+	// Cache player colors for performance
 	PlayerColorMap playerColors;
 
 	// Clear old selection rings
@@ -218,6 +248,62 @@ QUERYHANDLER(GetObjectSettings)
 	msg->settings = settings;
 }
 
+QUERYHANDLER(GetObjectMapSettings)
+{
+	std::vector<entity_id_t> ids = *msg->ids;
+
+	CmpPtr<ICmpTemplateManager> cmpTemplateManager(*g_Game->GetSimulation2(), SYSTEM_ENTITY);
+	ENSURE(cmpTemplateManager);
+
+	XMLWriter_File exampleFile;
+	{
+		XMLWriter_Element entitiesTag(exampleFile, "Entities");
+		{
+			for (entity_id_t id : ids)
+			{
+				XMLWriter_Element entityTag(exampleFile, "Entity");
+				{
+					//Template name
+					entityTag.Setting("Template", cmpTemplateManager->GetCurrentTemplateName(id));
+
+					//Player
+					CmpPtr<ICmpOwnership> cmpOwnership(*g_Game->GetSimulation2(), id);
+					if (cmpOwnership)
+						entityTag.Setting("Player", static_cast<int>(cmpOwnership->GetOwner()));
+
+					//Adding position to make some relative position later
+					CmpPtr<ICmpPosition> cmpPosition(*g_Game->GetSimulation2(), id);
+					if (cmpPosition)
+					{
+						CFixedVector3D pos = cmpPosition->GetPosition();
+						CFixedVector3D rot = cmpPosition->GetRotation();
+						{
+							XMLWriter_Element positionTag(exampleFile, "Position");
+							positionTag.Attribute("x", pos.X);
+							positionTag.Attribute("z", pos.Z);
+							// TODO: height offset etc
+						}
+						{
+							XMLWriter_Element orientationTag(exampleFile, "Orientation");
+							orientationTag.Attribute("y", rot.Y);
+							// TODO: X, Z maybe
+						}
+					}
+
+					// Adding actor seed
+					CmpPtr<ICmpVisual> cmpVisual(*g_Game->GetSimulation2(), id);
+					if (cmpVisual)
+						entityTag.Setting("ActorSeed", static_cast<unsigned int>(cmpVisual->GetActorSeed()));
+				}
+			}
+		}
+	}
+
+	const CStr& data = exampleFile.GetOutput();
+	msg->xmldata = data.FromUTF8();
+}
+
+
 BEGIN_COMMAND(SetObjectSettings)
 {
 	player_id_t m_PlayerOld, m_PlayerNew;
@@ -283,6 +369,7 @@ END_COMMAND(SetObjectSettings);
 
 static CStrW g_PreviewUnitName;
 static entity_id_t g_PreviewEntityID = INVALID_ENTITY;
+static std::vector<entity_id_t> g_PreviewEntitiesID;
 
 static CVector3D GetUnitPos(const Position& pos, bool floating)
 {
@@ -291,40 +378,156 @@ static CVector3D GetUnitPos(const Position& pos, bool floating)
 
 	// Clamp the position to the edges of the world:
 
-	// Use 'clamp' with a value slightly less than the width, so that converting
+	// Use 'Clamp' with a value slightly less than the width, so that converting
 	// to integer (rounding towards zero) will put it on the tile inside the edge
 	// instead of just outside
-	float mapWidth = (g_Game->GetWorld()->GetTerrain()->GetVerticesPerSide()-1)*TERRAIN_TILE_SIZE;
+	const float mapWidth = (g_Game->GetWorld()->GetTerrain().GetVerticesPerSide()-1)*TERRAIN_TILE_SIZE;
 	float delta = 1e-6f; // fraction of map width - must be > FLT_EPSILON
 
-	float xOnMap = clamp(vec.X, 0.f, mapWidth * (1.f - delta));
-	float zOnMap = clamp(vec.Z, 0.f, mapWidth * (1.f - delta));
+	float xOnMap = Clamp(vec.X, 0.f, mapWidth * (1.f - delta));
+	float zOnMap = Clamp(vec.Z, 0.f, mapWidth * (1.f - delta));
 
 	// Don't waste time with GetExactGroundLevel unless we've changed
 	if (xOnMap != vec.X || zOnMap != vec.Z)
 	{
 		vec.X = xOnMap;
 		vec.Z = zOnMap;
-		vec.Y = g_Game->GetWorld()->GetTerrain()->GetExactGroundLevel(xOnMap, zOnMap);
+		vec.Y = g_Game->GetWorld()->GetTerrain().GetExactGroundLevel(xOnMap, zOnMap);
 	}
 
 	return vec;
 }
 
+QUERYHANDLER(GetCurrentSelection)
+{
+	msg->ids = g_Selection;
+}
+
+MESSAGEHANDLER(ObjectPreviewToEntity)
+{
+	UNUSED2(msg);
+
+	if (g_PreviewEntitiesID.size() == 0)
+		return;
+
+	CmpPtr<ICmpTemplateManager> cmpTemplateManager(*g_Game->GetSimulation2(), SYSTEM_ENTITY);
+	ENSURE(cmpTemplateManager);
+
+	PlayerColorMap playerColor;
+
+	//I need to re create the objects finally delete preview objects
+	for (entity_id_t ent : g_PreviewEntitiesID)
+	{
+		//Get template name (without the "preview|" prefix)
+		std::wstring wTemplateName = wstring_from_utf8(cmpTemplateManager->GetCurrentTemplateName(ent).substr(8));
+		//Create new entity
+		entity_id_t new_ent = g_Game->GetSimulation2()->AddEntity(wTemplateName);
+		if (new_ent == INVALID_ENTITY)
+			continue;
+
+		//get position, get rotation
+		CmpPtr<ICmpPosition> cmpPositionNew(*g_Game->GetSimulation2(), new_ent);
+		CmpPtr<ICmpPosition> cmpPositionOld(*g_Game->GetSimulation2(), ent);
+
+		if (cmpPositionNew && cmpPositionOld)
+		{
+			CVector3D pos = cmpPositionOld->GetPosition();
+			cmpPositionNew->JumpTo(entity_pos_t::FromFloat(pos.X), entity_pos_t::FromFloat(pos.Z));
+
+			//now rotate
+			CFixedVector3D rotation = cmpPositionOld->GetRotation();
+			cmpPositionNew->SetYRotation(rotation.Y);
+		}
+
+		//get owner
+		CmpPtr<ICmpOwnership> cmpOwnershipNew(*g_Game->GetSimulation2(), new_ent);
+		CmpPtr<ICmpOwnership> cmpOwnershipOld(*g_Game->GetSimulation2(), ent);
+		if (cmpOwnershipNew && cmpOwnershipOld)
+			cmpOwnershipNew->SetOwner(cmpOwnershipOld->GetOwner());
+
+		//getVisual
+		CmpPtr<ICmpVisual> cmpVisualNew(*g_Game->GetSimulation2(), new_ent);
+		CmpPtr<ICmpVisual> cmpVisualOld(*g_Game->GetSimulation2(), ent);
+		if (cmpVisualNew && cmpVisualOld)
+			cmpVisualNew->SetActorSeed(cmpVisualOld->GetActorSeed());
+
+		//Update g_selectedObject and higligth
+		g_Selection.push_back(new_ent);
+		CmpPtr<ICmpSelectable> cmpSelectable(*g_Game->GetSimulation2(), new_ent);
+		if (cmpSelectable)
+			cmpSelectable->SetSelectionHighlight(GetOwnerPlayerColor(playerColor, new_ent), true);
+
+		g_Game->GetSimulation2()->DestroyEntity(ent);
+	}
+	g_PreviewEntitiesID.clear();
+
+}
+
+MESSAGEHANDLER(MoveObjectPreview)
+{
+	if (g_PreviewEntitiesID.size()==0)
+		return;
+
+	//TODO:Change pivot
+	entity_id_t referenceEntity = *g_PreviewEntitiesID.begin();
+
+	// All selected objects move relative to a pivot object,
+	//	so get its position and whether it's floating
+	CFixedVector3D referencePos;
+
+	CmpPtr<ICmpPosition> cmpPosition(*g_Game->GetSimulation2(), referenceEntity);
+	if (cmpPosition && cmpPosition->IsInWorld())
+		referencePos = cmpPosition->GetPosition();
+
+
+	// Calculate directional vector of movement for pivot object,
+	//	we apply the same movement to all objects
+	CVector3D targetPos = GetUnitPos(msg->pos, true);
+	CFixedVector3D fTargetPos(entity_pos_t::FromFloat(targetPos.X), entity_pos_t::FromFloat(targetPos.Y), entity_pos_t::FromFloat(targetPos.Z));
+	CFixedVector3D dir = fTargetPos - referencePos;
+
+	for (const entity_id_t id : g_PreviewEntitiesID)
+	{
+		CmpPtr<ICmpPosition> cmpPreviewPosition(*g_Game->GetSimulation2(), id);
+		if (cmpPreviewPosition)
+		{
+			CFixedVector3D posFinal;
+			if (cmpPreviewPosition->IsInWorld())
+			{
+				// Calculate this object's position
+				CFixedVector3D posFixed = cmpPreviewPosition->GetPosition();
+				posFinal = posFixed + dir;
+			}
+			cmpPreviewPosition->JumpTo(posFinal.X, posFinal.Z);
+		}
+
+		CheckObstructionAndUpdateVisual(id);
+	}
+}
+
 MESSAGEHANDLER(ObjectPreview)
 {
 	// If the selection has changed...
-	if (*msg->id != g_PreviewUnitName)
+	if (*msg->id != g_PreviewUnitName || (!msg->cleanObjectPreviews))
 	{
 		// Delete old entity
-		if (g_PreviewEntityID != INVALID_ENTITY)
-			g_Game->GetSimulation2()->DestroyEntity(g_PreviewEntityID);
+		if (g_PreviewEntityID != INVALID_ENTITY && msg->cleanObjectPreviews)
+		{
+			//Time to delete all preview objects
+			for (entity_id_t ent : g_PreviewEntitiesID)
+				g_Game->GetSimulation2()->DestroyEntity(ent);
+			g_PreviewEntitiesID.clear();
+		}
 
 		// Create the new entity
 		if ((*msg->id).empty())
 			g_PreviewEntityID = INVALID_ENTITY;
 		else
+		{
 			g_PreviewEntityID = g_Game->GetSimulation2()->AddLocalEntity(L"preview|" + *msg->id);
+			g_PreviewEntitiesID.push_back(g_PreviewEntityID);
+		}
+
 
 		g_PreviewUnitName = *msg->id;
 	}
@@ -336,7 +539,7 @@ MESSAGEHANDLER(ObjectPreview)
 		CmpPtr<ICmpPosition> cmpPosition(*g_Game->GetSimulation2(), g_PreviewEntityID);
 		if (cmpPosition)
 		{
-			CVector3D pos = GetUnitPos(msg->pos, cmpPosition->IsFloating());
+			CVector3D pos = GetUnitPos(msg->pos, cmpPosition->CanFloat());
 			cmpPosition->JumpTo(entity_pos_t::FromFloat(pos.X), entity_pos_t::FromFloat(pos.Z));
 
 			float angle;
@@ -363,6 +566,8 @@ MESSAGEHANDLER(ObjectPreview)
 		CmpPtr<ICmpOwnership> cmpOwnership(*g_Game->GetSimulation2(), g_PreviewEntityID);
 		if (cmpOwnership)
 			cmpOwnership->SetOwner((player_id_t)msg->settings->player);
+
+		CheckObstructionAndUpdateVisual(g_PreviewEntityID);
 	}
 }
 
@@ -377,7 +582,7 @@ BEGIN_COMMAND(CreateObject)
 	void Do()
 	{
 		// Calculate the position/orientation to create this unit with
-		
+
 		m_Pos = GetUnitPos(msg->pos, true); // don't really care about floating
 
 		if (msg->usetarget)
@@ -439,20 +644,19 @@ QUERYHANDLER(PickObject)
 {
 	float x, y;
 	msg->pos->GetScreenSpace(x, y);
-	
+
 	// Normally this function would be called with a player ID to check LOS,
 	//	but in Atlas the entire map is revealed, so just pass INVALID_PLAYER
-	std::vector<entity_id_t> ents = EntitySelection::PickEntitiesAtPoint(*g_Game->GetSimulation2(), *g_Game->GetView()->GetCamera(), x, y, INVALID_PLAYER, msg->selectActors);
+	entity_id_t ent = EntitySelection::PickEntityAtPoint(*g_Game->GetSimulation2(), *g_Game->GetView()->GetCamera(), x, y, INVALID_PLAYER, msg->selectActors);;
 
-	// Multiple entities may have been picked, but they are sorted by distance,
-	//	so only take the first one
-	if (!ents.empty())
+	if (ent == INVALID_ENTITY)
+		msg->id = INVALID_ENTITY;
+	else
 	{
-		msg->id = ents[0];
-
+		msg->id = ent;
 		// Calculate offset of object from original mouse click position
 		//	so it gets moved by that offset
-		CmpPtr<ICmpPosition> cmpPosition(*g_Game->GetSimulation2(), (entity_id_t)ents[0]);
+		CmpPtr<ICmpPosition> cmpPosition(*g_Game->GetSimulation2(), ent);
 		if (!cmpPosition || !cmpPosition->IsInWorld())
 		{
 			// error
@@ -463,18 +667,12 @@ QUERYHANDLER(PickObject)
 			CFixedVector3D fixed = cmpPosition->GetPosition();
 			CVector3D centre = CVector3D(fixed.X.ToFloat(), fixed.Y.ToFloat(), fixed.Z.ToFloat());
 
-			float cx, cy;
-			g_Game->GetView()->GetCamera()->GetScreenCoordinates(centre, cx, cy);
+			const CVector2D screenPos{g_Game->GetView()->GetCamera()->GetScreenCoordinates(centre)};
 
-			msg->offsetx = (int)(cx - x);
-			msg->offsety = (int)(cy - y);
+			msg->offsetx = static_cast<int>(screenPos.X - x);
+			msg->offsety = static_cast<int>(screenPos.Y - y);
 		}
 	}
-	else
-	{
-		// No entity picked
-		msg->id = INVALID_ENTITY;
-	}
 }
 
 
@@ -506,12 +704,22 @@ QUERYHANDLER(PickSimilarObjects)
 	msg->ids = EntitySelection::PickSimilarEntities(*g_Game->GetSimulation2(), *g_Game->GetView()->GetCamera(), templateName, owner, false, true, true, false);
 }
 
+MESSAGEHANDLER(ResetSelectionColor)
+{
+	UNUSED2(msg);
+
+	for (entity_id_t ent : g_Selection)
+	{
+		CmpPtr<ICmpVisual> cmpVisual(*g_Game->GetSimulation2(), ent);
+		if (cmpVisual)
+			cmpVisual->SetShadingColor(fixed::FromDouble(1), fixed::FromDouble(1), fixed::FromDouble(1), fixed::FromDouble(1));
+	}
+}
 
 BEGIN_COMMAND(MoveObjects)
 {
 	// Mapping from object to position
-	typedef std::map<entity_id_t, CVector3D> ObjectPositionMap;
-	ObjectPositionMap m_PosOld, m_PosNew;
+	std::map<entity_id_t, CVector3D> m_PosOld, m_PosNew;
 
 	void Do()
 	{
@@ -522,11 +730,11 @@ BEGIN_COMMAND(MoveObjects)
 		CVector3D pivotPos(0, 0, 0);
 		bool pivotFloating = false;
 
-		CmpPtr<ICmpPosition> cmpPosition(*g_Game->GetSimulation2(), (entity_id_t)msg->pivot);
-		if (cmpPosition && cmpPosition->IsInWorld())
+		CmpPtr<ICmpPosition> cmpPositionPivot(*g_Game->GetSimulation2(), (entity_id_t)msg->pivot);
+		if (cmpPositionPivot && cmpPositionPivot->IsInWorld())
 		{
-			pivotFloating = cmpPosition->IsFloating();
-			CFixedVector3D pivotFixed = cmpPosition->GetPosition();
+			pivotFloating = cmpPositionPivot->CanFloat();
+			CFixedVector3D pivotFixed = cmpPositionPivot->GetPosition();
 			pivotPos = CVector3D(pivotFixed.X.ToFloat(), pivotFixed.Y.ToFloat(), pivotFixed.Z.ToFloat());
 		}
 
@@ -535,9 +743,8 @@ BEGIN_COMMAND(MoveObjects)
 		CVector3D targetPos = GetUnitPos(msg->pos, pivotFloating);
 		CVector3D dir = targetPos - pivotPos;
 
-		for (size_t i = 0; i < ids.size(); ++i)
+		for (entity_id_t id : ids)
 		{
-			entity_id_t id = (entity_id_t)ids[i];
 			CmpPtr<ICmpPosition> cmpPosition(*g_Game->GetSimulation2(), id);
 			if (!cmpPosition || !cmpPosition->IsInWorld())
 			{
@@ -550,25 +757,25 @@ BEGIN_COMMAND(MoveObjects)
 				CFixedVector3D posFixed = cmpPosition->GetPosition();
 				CVector3D pos = CVector3D(posFixed.X.ToFloat(), posFixed.Y.ToFloat(), posFixed.Z.ToFloat());
 				m_PosNew[id] = pos + dir;
-				m_PosOld[id] = pos;			
+				m_PosOld[id] = pos;
 			}
 		}
 
 		SetPos(m_PosNew);
 	}
 
-	void SetPos(ObjectPositionMap& map)
+	void SetPos(const std::map<entity_id_t, CVector3D>& map)
 	{
-		ObjectPositionMap::iterator it;
-		for (it = map.begin(); it != map.end(); ++it)
+		for (const std::pair<const entity_id_t, CVector3D>& p : map)
 		{
-			CmpPtr<ICmpPosition> cmpPosition(*g_Game->GetSimulation2(), (entity_id_t)it->first);
+			CmpPtr<ICmpPosition> cmpPosition(*g_Game->GetSimulation2(), p.first);
 			if (!cmpPosition)
 				return;
 
 			// Set 2D position, ignoring height
-			CVector3D pos = it->second;
-			cmpPosition->JumpTo(entity_pos_t::FromFloat(pos.X), entity_pos_t::FromFloat(pos.Z));
+			cmpPosition->JumpTo(entity_pos_t::FromFloat(p.second.X), entity_pos_t::FromFloat(p.second.Z));
+
+			CheckObstructionAndUpdateVisual(p.first);
 		}
 	}
 
@@ -591,40 +798,178 @@ BEGIN_COMMAND(MoveObjects)
 };
 END_COMMAND(MoveObjects)
 
-
-BEGIN_COMMAND(RotateObject)
+BEGIN_COMMAND(RotateObjectsFromCenterPoint)
 {
-	float m_AngleOld, m_AngleNew;
+	std::map<entity_id_t, CVector3D> m_PosOld, m_PosNew;
+	std::map<entity_id_t, float> m_AngleOld, m_AngleNew;
+	CVector3D m_CenterPoint;
+	float m_AngleInitialRotation;
 
 	void Do()
 	{
-		CmpPtr<ICmpPosition> cmpPosition(*g_Game->GetSimulation2(), (entity_id_t)msg->id);
-		if (!cmpPosition)
-			return;
+		std::vector<entity_id_t> ids = *msg->ids;
 
-		m_AngleOld = cmpPosition->GetRotation().Y.ToFloat();
-		if (msg->usetarget)
+		CVector3D minPos;
+		CVector3D maxPos;
+
+		bool first = true;
+
+		// Compute min position and max position
+		for (entity_id_t id : ids)
 		{
-			CMatrix3D transform = cmpPosition->GetInterpolatedTransform(0.f, false);
+			CmpPtr<ICmpPosition> cmpPosition(*g_Game->GetSimulation2(), id);
+			if (!cmpPosition)
+				continue;
+
+			CVector3D pos = cmpPosition->GetPosition();
+
+			m_PosOld[id] = cmpPosition->GetPosition();
+			m_AngleOld[id] = cmpPosition->GetRotation().Y.ToFloat();
+
+			if (first)
+			{
+				first = false;
+				minPos = pos;
+				maxPos = pos;
+				m_CenterPoint.Y = pos.Y;
+				continue;
+			}
+
+			if (pos.X < minPos.X)
+				minPos.X = pos.X;
+
+			if (pos.X > maxPos.X)
+				maxPos.X = pos.X;
+
+			if (pos.Z < minPos.Z)
+				minPos.Z = pos.Z;
+
+			if (pos.Z > maxPos.Z)
+				maxPos.Z = pos.Z;
+		}
+
+		// Calculate objects center point
+		m_CenterPoint.X = minPos.X + ((maxPos.X - minPos.X) * 0.5);
+		m_CenterPoint.Z = minPos.Z + ((maxPos.Z - minPos.Z) * 0.5);
+
+		CVector3D target = msg->target->GetWorldSpace(m_CenterPoint.Y);
+		m_AngleInitialRotation = atan2(target.X-m_CenterPoint.X, target.Z-m_CenterPoint.Z);
+	}
+
+	void SetPos(const std::map<entity_id_t, CVector3D>& position, const std::map<entity_id_t, float>& angle)
+	{
+		for (const std::pair<const entity_id_t, CVector3D>& p : position)
+		{
+			CmpPtr<ICmpPosition> cmpPosition(*g_Game->GetSimulation2(), p.first);
+			if (!cmpPosition)
+				return;
+
+			// Set 2D position, ignoring height
+			cmpPosition->JumpTo(entity_pos_t::FromFloat(p.second.X), entity_pos_t::FromFloat(p.second.Z));
+
+			if (msg->rotateObject)
+				cmpPosition->SetYRotation(entity_angle_t::FromFloat(angle.at(p.first)));
+
+		}
+
+		for (const std::pair<const entity_id_t, CVector3D>& p : position)
+			CheckObstructionAndUpdateVisual(p.first);
+	}
+
+	void Redo()
+	{
+		SetPos(m_PosNew, m_AngleNew);
+	}
+
+	void RecalculateRotation(Position newPoint)
+	{
+		std::vector<entity_id_t> ids = *msg->ids;
+
+		CVector3D target = newPoint.GetWorldSpace(m_CenterPoint.Y);
+		float newAngle = atan2(target.X-m_CenterPoint.X, target.Z-m_CenterPoint.Z);
+
+		float globalAngle = m_AngleInitialRotation - newAngle;
+
+		// Recalculate positions
+		for (entity_id_t id : ids)
+		{
+			CVector3D pos = m_PosOld[id];
+			float angle = atan2(pos.X - m_CenterPoint.X, pos.Z - m_CenterPoint.Z);
+			float localAngle = angle + (globalAngle - angle);
+			float xCos = cosf(localAngle);
+			float xSin = sinf(localAngle);
+
+			pos.X -= m_CenterPoint.X;
+			pos.Z -= m_CenterPoint.Z;
+
+			float newX = pos.X * xCos - pos.Z * xSin;
+			float newZ = pos.X * xSin + pos.Z * xCos;
+
+			pos.X = newX + m_CenterPoint.X;
+			pos.Z = newZ + m_CenterPoint.Z;
+
+			m_PosNew[id] = pos;
+
+			m_AngleNew[id] = m_AngleOld[id] - globalAngle;
+		}
+
+		SetPos(m_PosNew, m_AngleNew);
+	}
+
+	void Undo()
+	{
+		SetPos(m_PosOld, m_AngleOld);
+	}
+
+	void MergeIntoPrevious(cRotateObjectsFromCenterPoint* prev)
+	{
+		// TODO: do something valid if prev unit != this unit
+		ENSURE(*prev->msg->ids == *msg->ids);
+		m_PosOld = prev->m_PosOld;
+		m_AngleInitialRotation = prev->m_AngleInitialRotation;
+		m_AngleOld = prev->m_AngleOld;
+		m_CenterPoint = prev->m_CenterPoint;
+
+		RecalculateRotation(msg->target);
+	}
+};
+END_COMMAND(RotateObjectsFromCenterPoint)
+
+BEGIN_COMMAND(RotateObject)
+{
+	std::map<entity_id_t, float> m_AngleOld, m_AngleNew;
+
+	void Do()
+	{
+		std::vector<entity_id_t> ids = *msg->ids;
+
+		for (entity_id_t id : ids)
+		{
+			CmpPtr<ICmpPosition> cmpPosition(*g_Game->GetSimulation2(), id);
+			if (!cmpPosition)
+				return;
+
+			m_AngleOld[id] = cmpPosition->GetRotation().Y.ToFloat();
+
+			CMatrix3D transform = cmpPosition->GetInterpolatedTransform(0.f);
 			CVector3D pos = transform.GetTranslation();
 			CVector3D target = msg->target->GetWorldSpace(pos.Y);
-			m_AngleNew = atan2(target.X-pos.X, target.Z-pos.Z);
-		}
-		else
-		{
-			m_AngleNew = msg->angle;
+			m_AngleNew[id] = atan2(target.X-pos.X, target.Z-pos.Z);
 		}
 
 		SetAngle(m_AngleNew);
 	}
 
-	void SetAngle(float angle)
+	void SetAngle(const std::map<entity_id_t, float>& angles)
 	{
-		CmpPtr<ICmpPosition> cmpPosition(*g_Game->GetSimulation2(), (entity_id_t)msg->id);
-		if (!cmpPosition)
-			return;
+		for (const std::pair<const entity_id_t, float>& p : angles)
+		{
+			CmpPtr<ICmpPosition> cmpPosition(*g_Game->GetSimulation2(), p.first);
+			if (!cmpPosition)
+				return;
 
-		cmpPosition->SetYRotation(entity_angle_t::FromFloat(angle));
+			cmpPosition->SetYRotation(entity_angle_t::FromFloat(p.second));
+		}
 	}
 
 	void Redo()
@@ -640,7 +985,7 @@ BEGIN_COMMAND(RotateObject)
 	void MergeIntoPrevious(cRotateObject* prev)
 	{
 		// TODO: do something valid if prev unit != this unit
-		ENSURE(prev->msg->id == msg->id);
+		ENSURE(*prev->msg->ids == *msg->ids);
 		prev->m_AngleNew = m_AngleNew;
 	}
 };
@@ -654,9 +999,10 @@ BEGIN_COMMAND(DeleteObjects)
 	{
 		entity_id_t entityID;
 		CStr templateName;
-		int32_t owner;
+		player_id_t owner;
 		CFixedVector3D pos;
 		CFixedVector3D rot;
+		u32 actorSeed;
 	};
 
 	std::vector<OldObject> oldObjects;
@@ -695,9 +1041,14 @@ BEGIN_COMMAND(DeleteObjects)
 				obj.rot = cmpPosition->GetRotation();
 			}
 
+			CmpPtr<ICmpVisual> cmpVisual(sim, obj.entityID);
+			if (cmpVisual)
+				obj.actorSeed = cmpVisual->GetActorSeed();
+
 			oldObjects.push_back(obj);
 			g_Game->GetSimulation2()->DestroyEntity(obj.entityID);
 		}
+		g_Game->GetSimulation2()->FlushDestroyedEntities();
 	}
 
 	void Undo()
@@ -709,7 +1060,7 @@ BEGIN_COMMAND(DeleteObjects)
 			entity_id_t ent = sim.AddEntity(oldObjects[i].templateName.FromUTF8(), oldObjects[i].entityID);
 			if (ent == INVALID_ENTITY)
 			{
-				LOGERROR(L"Failed to load entity template '%hs'", oldObjects[i].templateName.c_str());
+				LOGERROR("Failed to load entity template '%s'", oldObjects[i].templateName.c_str());
 			}
 			else
 			{
@@ -724,6 +1075,10 @@ BEGIN_COMMAND(DeleteObjects)
 				CmpPtr<ICmpOwnership> cmpOwnership(sim, oldObjects[i].entityID);
 				if (cmpOwnership)
 					cmpOwnership->SetOwner(oldObjects[i].owner);
+
+				CmpPtr<ICmpVisual> cmpVisual(sim, oldObjects[i].entityID);
+				if (cmpVisual)
+					cmpVisual->SetActorSeed(oldObjects[i].actorSeed);
 			}
 		}
 
@@ -754,4 +1109,23 @@ MESSAGEHANDLER(SetBandbox)
 	AtlasView::GetView_Game()->SetBandbox(msg->show, (float)msg->sx0, (float)msg->sy0, (float)msg->sx1, (float)msg->sy1);
 }
 
+QUERYHANDLER(GetSelectedObjectsTemplateNames)
+{
+	std::vector<entity_id_t> ids = *msg->ids;
+	std::vector<std::string> names;
+
+	CmpPtr<ICmpTemplateManager> cmpTemplateManager(*g_Game->GetSimulation2(), SYSTEM_ENTITY);
+	ENSURE(cmpTemplateManager);
+
+	for (size_t i = 0; i < ids.size(); ++i)
+	{
+		entity_id_t id = (entity_id_t)ids[i];
+		std::string templateName = cmpTemplateManager->GetCurrentTemplateName(id);
+		names.push_back(templateName);
+	}
+
+	std::sort(names.begin(), names.end());
+	msg->names = names;
 }
+
+} // namespace AtlasMessage
